#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from ..outils.Arithmetique import pgcd, ppcm, premier, factoriseTex, carrerise
from ..outils.Affichage import decimaux
from random import randint, shuffle

premiers = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61,
        67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139,
        149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223,
        227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293,
        307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383,
        389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463,
        467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557, 563, 569,
        571, 577, 587, 593, 599, 601, 607, 613, 617, 619, 631, 641, 643, 647,
        653, 659, 661, 673, 677, 683, 691, 701, 709, 719, 727, 733, 739, 743,
        751, 757, 761, 769, 773, 787, 797, 809, 811, 821, 823, 827, 829, 839,
        853, 857, 859, 863, 877, 881, 883, 887, 907, 911, 919, 929, 937, 941,
        947, 953, 967, 971, 977, 983, 991, 997] ## Liste de longueur 168

def Arithmetique():
  """Exercice de décomposition de nombres en facteurs premiers, puis de
  recherche du PGCD et du PPCM, et d'applications aux fractions"""

  ### Question 1
  exo = ["\\exercice", '\\begin{enumerate}', u"\\item Donner la décomposition" +
          u" en facteurs premiers des nombres suivants, et préciser quand il" +
          u" s\'agit d\'un nombre premier :\\par"]
  cor = ["\\exercice*", '\\begin{enumerate}', u"\\item Donner la décomposition"
          + u" en facteurs premiers des nombres suivants, et préciser quand il"
          + u" s\'agit d\'un nombre premier :\\par"]

  prime = premiers[randint(10,167)]

  fauxpgcd = randint(1,101)
  fauxpgcdfactor = factoriseTex(fauxpgcd)[0]

  complementaires = [randint(6,50), randint(6,50)]
  pgcdcompl = pgcd(complementaires[0], complementaires[1])
  pgcdcomplfact = factoriseTex(pgcdcompl)[0]

  if pgcdcompl != 1:
    facteurs = pgcdcomplfact + fauxpgcdfactor
    facteurs.sort()
  else:
    facteurs = fauxpgcdfactor

  autresnombres = [randint(51,999), randint(51,999)]

  listenombres = [prime, complementaires[0] * fauxpgcd, complementaires[1] *
          fauxpgcd, ] + autresnombres
  melange = [prime, complementaires[0] * fauxpgcd, complementaires[1] *
          fauxpgcd, ] + autresnombres
  shuffle(melange)

  exo.append(decimaux(melange[0]) + " ; " + decimaux(melange[1]) + " ; " +
          decimaux(melange[2]) + " ; " + decimaux(melange[3]) + " ; " +
          decimaux(melange[4]) + " ;")

  cor.append("\\begin{multicols}{2}")

  for i in range(5):
    cor += factoriseTex(melange[i])[1]

  cor.append("\\end{multicols}")

  ### Question 2
  exo.append(u'\\item En déduire le PGCD et le PPCM des nombres ' +
          decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
          ".")
  cor.append(u'\\item En déduire le PGCD et le PPCM des nombres ' +
          decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
          ".\\par")

  cor.append(u"D'après la question 1), on sait que les nombres " +
          decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
          " ont comme facteurs premiers communs : ")

  for j in range(len(facteurs)):
    if j == 0:
      temp = "$"
    if j != len(facteurs)-1:
      temp += decimaux(facteurs[j]) + " , "
    else:
      temp += decimaux(facteurs[j]) + "$.\\par"
  cor.append(temp)

  cor.append(u"On en déduit que le PGCD des nombres " +
          decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
          " est : ")
  temp = "$"
  if len(facteurs) > 1:
    for j in range(len(facteurs)):
      if j != len(facteurs)-1:
        temp += decimaux(facteurs[j]) + " \\times "
      else:
        temp += decimaux(facteurs[j]) + " = "

  temp+= decimaux(fauxpgcd * pgcdcompl) + ".$\\par"
  cor.append(temp)

  vraippcm = (listenombres[1] * listenombres[2]) / (fauxpgcd * pgcdcompl)

  if (listenombres[1] % listenombres[2] == 0):
    cor.append(decimaux(listenombres[1]) + u" est un multiple de " +
            decimaux(listenombres[2]) + u", donc leur PPCM est directement "
               + decimaux(listenombres[1]) + ".")
  elif (listenombres[2] % listenombres[1] == 0):
    cor.append(decimaux(listenombres[2]) + u" est un multiple de " +
            decimaux(listenombres[1]) + u", donc leur PPCM est directement " +
            decimaux(listenombres[2]) + ".")
  else:
    cor.append(u"Il existe plusieurs méthodes pour calculer le PPCM de " +
            decimaux(listenombres[1]) + " et de " + decimaux(listenombres[2]) +
            ".\\par" )
    cor.append(u"En voici deux :")
    cor.append("\\begin{enumerate}")

    cor.append(u"\\item On peut simplement utiliser la formule :")
    cor.append(u"$a \\times b = PGCD(a;~b) \\times PPCM(a;~b)$.\\par")
    cor.append(u"Donc : $PPCM(" + decimaux(listenombres[1]) + ";~" +
        decimaux(listenombres[2]) + ") = " + "\\dfrac{" +
        decimaux(listenombres[1]) + "\\times" + decimaux(listenombres[2]) + "}{"
        + decimaux(fauxpgcd * pgcdcompl) + "} = " + decimaux(vraippcm) +
        ".$")

    cor.append(u"\\item On peut aussi multiplier un nombre par les \"facteurs "
        + u"complémentaires\" de l'autre.\n" + u"Ces \"facteurs " +
        u"complémentaires\" sont les facteurs qui complètent le PGCD pour " +
        u"former le nombre.\\par")

    temp = u"Comme $PGCD(" + decimaux(listenombres[1]) + ";~" +\
            decimaux(listenombres[2]) + ") = " + decimaux(fauxpgcd * pgcdcompl)

    if len(facteurs) > 1:
      temp += " = "
      for j in range(len(facteurs)):
        if j != len(facteurs)-1:
          temp += decimaux(facteurs[j]) + " \\times "
        else:
          temp += decimaux(facteurs[j])
    
    factornb1 = factoriseTex(listenombres[1])[0]  
    
    if len(factornb1) > 1:
      textcompl = u"$, alors les \"facteurs complémentaires\" de $"
    else:
      textcompl = u"$, alors le \"facteur complémentaire\" de $"

    temp += textcompl + decimaux(listenombres[1]) + " = "

    for j in range(len(factornb1)):
      if j != len(factornb1)-1:
        temp += decimaux(factornb1[j]) + " \\times "
      else:
        temp += decimaux(factornb1[j])
        
    factcompl = factoriseTex(listenombres[1] / (fauxpgcd * pgcdcompl))[0]

    if len(factcompl) == 1:
      temp += u"$ est : "
    else:
      temp += u"$ sont : "
    for j in range(len(factcompl)):
      if j != len(factcompl)-1:
        temp += decimaux(factcompl[j]) + " , "
      else:
        temp += decimaux(factcompl[j]) + ".\n"
    temp += u"On en déduit que $PPCM(" + decimaux(listenombres[1]) + ";~" +\
            decimaux(listenombres[2]) + ") = " + decimaux(listenombres[2]) +\
            " \\times "

    for j in range(len(factcompl)):
      if j != len(factcompl)-1:
        temp += decimaux(factcompl[j]) + " \\times "
      else:
        temp += decimaux(factcompl[j]) + " = "
    temp += decimaux(vraippcm) + ".$"
    cor.append(temp)
    cor.append("\\end{enumerate}")

  ### Question 3

  exo.append(u"\\item Quel est le plus petit nombre par lequel il faut " +
          u"multiplier " + decimaux(autresnombres[0]) +
          u" pour obtenir un carré parfait ?")

  cor.append(u" \\item Pour obtenir un carré parfait, il faut que sa " +
          u"décomposition en facteurs premiers ne contienne que des facteurs "
          + u"apparaissant un nombre pair de fois. D'après la question 1, " +
          u"la décomposition en facteurs premiers de "
          + decimaux(autresnombres[0]))

  decompautre = factoriseTex(autresnombres[0])[1]

  if len(decompautre) == 1:
    cor.append(u" est lui-même, car c'est un nombre premier.")
  else:
    cor.append(" est : \\par\n$" + decimaux(autresnombres[0]) + " = " +
            decompautre[-2][5:-2] + ".$\\par")

  cor.append(u"Il faut donc encore multiplier ce nombre par ")

  carre = carrerise(autresnombres[0])
  factsup = factoriseTex(carre)[0]

  if len(factsup)==1:
    cor.append(" le facteur ")
  else:
    cor.append(" les facteurs ")

  for j in range(len(factsup)):
    if (j != len(factsup)-1) and (j != len(factsup)-2):
      cor.append(decimaux(factsup[j]) + " , ")
    elif (j == len(factsup)-2):
      cor.append(decimaux(factsup[j]) + " et ")
    else:
      cor.append(decimaux(factsup[j]) + ".\\par")

  cor.append(u"Le nombre cherché est par conséquent " + decimaux(carre) +
          u" et le carré parfait obtenu est " + decimaux(carre *
              autresnombres[0]) + ".")

  ### Question 4
  exo.append(u"\\item Rendre la fraction $\\dfrac{" + decimaux(listenombres[1])
          + "}{" + decimaux(listenombres[2]) + u"}$ irréductible.")

  cor.append(u"\\item Le moyen le plus rapide de simplifier cette fraction est"
          + u"de diviser le numérateur et le dénominateur par leur PGCD." +
          u" D'après la question 2),  PGCD(" + decimaux(listenombres[1]) + ";~"
          + decimaux(listenombres[2]) + ") = "
          + decimaux(fauxpgcd * pgcdcompl) + ", donc on obtient :\\par")
  cor.append(u"$\dfrac{" + decimaux(listenombres[1]) + "{\\scriptstyle \\div " +
          decimaux(fauxpgcd * pgcdcompl) + "}}{" + decimaux(listenombres[2]) +
          "{\\scriptstyle \\div " + decimaux(fauxpgcd * pgcdcompl) +
          "}} = \dfrac{" + decimaux(listenombres[1] / (fauxpgcd * pgcdcompl)) +
          "}{" + decimaux(listenombres[2] / (fauxpgcd * pgcdcompl)) + "}.$")

  ### Question 5

  num = [randint(6,50), randint(6,50)]
  exo.append(u"\\item Calculer $\\dfrac{" + decimaux(num[0]) + "}{" +
          decimaux(listenombres[1]) + "} + \\dfrac{" + decimaux(num[1]) + "}{" +
          decimaux(listenombres[2]) + "}$.")

  mult1 = vraippcm / listenombres[1]
  mult2 = vraippcm / listenombres[2]

  num1 = mult1 * num[0]
  num2 = mult2 * num[1]

  simplfin = pgcd(num1+num2,vraippcm)

  if simplfin != 1:
    simpl = "{\\scriptstyle \\div " + decimaux(simplfin) + "}"
    result = " = \\dfrac{" + decimaux((num1+num2)/simplfin) + "}{" + \
            decimaux((vraippcm)/simplfin) + "}"
  else:
    simpl = ""
    result = ""

  cor.append(u"\\item Il faut mettre les fractions au même dénominateur. Grâce"
          + u"à la question 2), nous avons déjà un dénominateur commun : " +
          u"le PPCM des nombres " + decimaux(listenombres[1]) + " et " +
          decimaux(listenombres[2]) + u", qui est par définition le plus petit"
          + u"multiple commun de ces deux nombres.\\par")
  cor.append(u"$\\dfrac{" + decimaux(num[0]) + "{\\scriptstyle \\times " +
          decimaux(mult1) + "}}{" + decimaux(listenombres[1]) +
          "{\\scriptstyle \\times " + decimaux(mult1) + "}} + \\dfrac{" +
          decimaux(num[1]) + "{\\scriptstyle \\times " + decimaux(mult2) + "}}{"
          + decimaux(listenombres[2]) + "{\\scriptstyle \\times " +
          decimaux(mult2) + "}} = \\dfrac{" + decimaux(num1) + "}{" +
          decimaux(vraippcm) + "} + \\dfrac{" + decimaux(num2) + "}{" +
          decimaux(vraippcm) + "} = \\dfrac{" + decimaux(num1+num2) + simpl +
          "}{" + decimaux(vraippcm) + simpl + "}" + result + ".$")

  exo.append("\\end{enumerate}")
  cor.append("\\end{enumerate}")

  return (exo,cor)



# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randint, shuffle, randrange
from ..outils.decimaux import decimaux
import math
from ..outils.Geometrie import cotation, cotation_h

def choix_valeurs(min_prct, max_prct, min_nbre, max_nbre):

    while 1:
        decomp100 = [2,2,5,5,1,1]
        shuffle(decomp100)
        prct = decomp100[0] * decomp100[1] * decomp100[2]  # qui donne 2, 4, 5, 10, 20, 25 ou 50
        tot = decomp100[3] * decomp100[4] * decomp100[5]
        if (max_prct / prct != 0) and (max_nbre / tot != 0):
            break
    prct = randint(max(min_prct / prct, 1 ), (max_prct / prct )) * prct
    tot = tot * randint( max(min_nbre / tot,1) , (max_nbre / tot ))
    return prct, tot
    
def proportionnalite_3eme():
    #choix exercice
    i = randrange(2)
    #choix valeurs
    [min_prct, max_prct, min_nbre, max_nbre] = [[40, 80, 40, 65],
                                                [40, 80, 65, 90]][i]
    #choisit les valeurs du groupe A
    #prctA : pourcentage d'individu dans le groupe A entre 40% et 80%
    #totA : nombre total du groupe A entre 40 et 70
    prctA, totA = choix_valeurs(min_prct = 40, max_prct = 80, min_nbre = 40, max_nbre = 65)

    #On choisit B avec des valeurs différentes
    while 1:
        prctB, totB = choix_valeurs(min_prct = 40, max_prct = 80, min_nbre = 40, max_nbre = 65)
        if prctA != prctB and totB != totA:
            break
        
    Total = totA + totB

    #calcul
    nbA = (prctA * totA / 100)
    nbB = (prctB * totB / 100)
    nbTotal = ((nbA + nbB))
    prct_final = (100.0*nbTotal/Total)
    prct_final_arrondi = round(100.0*nbTotal/Total, 1)
    if prct_final == prct_final_arrondi:
        approx = "="
    else:
        approx = "\\approx"
    
    #La situation
    texte = [u"Lors d'un voyage scolaire, les élèves sont répartis dans deux bus :",
             u"Au collège Dubois, il y a $%s$ élèves en quatrièmes et $%s$ élèves en troisièmes."%(totA,totB)][i]

    #Les deux groupes
    ligne1 = [u"\\item Bus A : %s élèves dont %s %s de garçons."%(totA, prctA,"\\%"),
              u"\\item $%s$ %s des élèves de quatrièmes possèdent un ordinateur ;"%(prctA,"\\%")][i]
    ligne2 = [u"\\item Bus A : %s élèves dont %s %s de garçons."%(totB, prctB,"\\%"),
              u"\\item $%s$ %s des élèves de troisièmes possèdent un ordinateur ;"%(prctB,"\\%")][i]

    #La question
    question = [u"Quel est, sur l'ensemble des deux bus, le pourcentage de garçons ?\\par",
                u"Quel est le pourcentage des élèves qui possèdent un ordinateur ?\\par"][i]

    #La correction
    correction =[#Dans le bus
        [u"Dans le bus A, il y a $\\cfrac{%s \\times %s}{100} = %s$ garçons.\\par"%(prctA, totA, decimaux(nbA)),
           u"Dans le bus B, il y a $\\cfrac{%s \\times %s}{100} = %s$ garçons.\\par"%(prctB, totB, decimaux(nbB)),
           u"On en déduit qu'il y a $%s + %s = %s$ garçons sur un total de $%s + %s = %s $ élèves.\\par"
                   %(decimaux(nbA), decimaux(nbB), nbTotal, totA, totB, Total),
            u" Le pourcentage de garçons sur l'ensemble des deux bus est donc de $\\cfrac{%s}{%s}\\times 100 %s %s%s$"
                  %(nbTotal, Total, approx, decimaux(prct_final_arrondi), "\,\\%")],
        #À l'ordinateurs
                 [u"En quatrièmes, $\\cfrac{%s \\times %s}{100} = %s$ élèves possèdent un ordinateur.\\par"%(prctA, totA, decimaux(nbA)),
                  u"En troisièmes, $\\cfrac{%s \\times %s}{100} = %s$ élèves possèdent un ordinateur.\\par"%(prctB, totB, decimaux(nbB)),
                  u"On en déduit qu'il y a $%s + %s = %s$ élèves qui possèdent un ordinateur sur un total de $%s + %s = %s $ élèves.\\par"
                   %(decimaux(nbA), decimaux(nbB), nbTotal, totA, totB, Total),
                   u" Le pourcentage d'élèves possédant un ordinateur est donc de $\\cfrac{%s}{%s}\\times 100 %s %s%s$"
                  %(nbTotal, Total, approx, decimaux(prct_final_arrondi), "\,\\%")],
                   ][i]
    exo = ["\\exercice",
           texte,
           "\\begin{itemize}",
           ligne1,
           ligne2,
           "\\end{itemize}",
           question,
           ]
        
    cor = ["\\exercice*",
           texte,
           "\\begin{itemize}",
           ligne1,
           ligne2,
           "\\end{itemize}",
           question,
           "\\dotfill\\par"
           ] + correction
    
    
    return (exo,cor)





            
              














#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from ..outils.Arithmetique import ecrit_tex, valeur_alea, signe
from random import randrange

#
# ------------------- DEVELOPPEMENTS -------------------


def exo_distributivite():
    exo = ["\\exercice",
           u"Développer et réduire les expressions suivantes :",
           "\\begin{multicols}{2}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Développer et réduire les expressions suivantes :",
           "\\begin{multicols}{2}", "  \\noindent%"]
    for i in range(8):
        (l1, l2) = tex_developpe1(valeurs_distr(10))
        exo.extend(l1)
        cor.extend(l2)
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)


def exo_double_distributivite():
    exo = ["\\exercice",
           u"Développer et réduire les expressions suivantes :",
           "\\begin{multicols}{2}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Développer et réduire les expressions suivantes :",
           "\\begin{multicols}{2}", "  \\noindent%"]
    for i in range(6):
        (l1, l2) = tex_developpe1(valeurs_dbldistr(10))
        exo.extend(l1)
        cor.extend(l2)
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)


def coef_opposes(a):  # renvoie un tuple dont les valeurs sont les opposees de celles de a
    l = []
    for i in range(len(a)):
        l.append(-a[i])
    return tuple(l)


def dev(a):  # renvoi un tuple avec les 3 coefficients du developpement
    return (a[0][0] * a[1][0], a[0][0] * a[1][1] + a[0][1] * a[1][0], a[0][1] *
            a[1][1])


def somme_polynomes(a, b):  # renvoie un tuple dont les valeurs sont les sommes des valeurs correspondantes dans a et b
    l = []
    if len(a) > len(b):
        long = len(a)
    else:
        long = len(b)
    for i in range(int):
        if (len(a) - i) - 1 < 0:
            l.append(b[(len(b) - 1) - i])
        elif (len(b) - i) - 1 < 0:
            l.append(a[(len(a) - 1) - i])
        else:
            l.append(a[(len(a) - 1) - i] + b[(len(b) - 1) - i])
    l.reverse()
    return tuple(l)


def tex_coef(coef, var, bplus=0, bpn=0, bpc=0):

    # coef est le coefficient à écrire devant la variable var
    # bplus est un booleen : s'il est vrai, il faut ecrire le signe +
    # bpn est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef est negatif.
    # bpc est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef =! 0 ou 1 et var est non vide

    if coef != 0 and abs(coef) != 1:
        if var == '':
            if abs(coef) >= 1000:
                a = '\\nombre{%s}' % coef
            else:
                a = '%s' % coef
        else:
            if abs(coef) >= 1000:
                a = '\\nombre{%s}\\,%s' % (coef, var)
            else:
                a = '%s\\,%s' % (coef, var)
        if bplus and coef > 0:
            a = '+' + a
    elif coef == 1:
        if var == '':
            a = '1'
        else:
            a = '%s' % var
        if bplus:
            a = '+' + a
    elif coef == 0:
        a = ''
    elif coef == -1:
        if var == '':
            a = '-1'
        else:
            a = '-%s' % var
    if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != '':
        a = '\\left( ' + a + '\\right)'
    return a


def tex_dev0(a, bplus=0):  # renvoi (a+b)², (a-b)² ou (a+b)(c+d) ou a(c+d) ou (a+b)*c

    # a est de la forme ((3, 2), (3, 2)) pour (3x+2)(3x+2)

    (ca, cb, cc, cd) = (a[0][0], a[0][1], a[1][0], a[1][1])  # coefficients a, b, c et d
    if ca == 0 and cb == 0 or cc == 0 and cd == 0:
        return '0'
    elif a[0] == a[1]:

                        # (a+b)² ou (a-b)²

        if ca == 0:
            return '%s^2' % tex_coef(cb, '', bpn=1)
        elif cb == 0:
            return '%s^2' % tex_coef(ca, 'x', bpc=1)
        else:
            return '(%s%s)^2' % (tex_coef(ca, 'x'), tex_coef(cb, '',
                                 bplus=1))
    else:

        # (a+b)(c+d)

        if ca == 0 or cb == 0:
            if cc == 0 or cd == 0:
                return '%s%s\\times %s%s' % (tex_coef(ca, 'x'), tex_coef(cb,
                        '', bplus=ca != 0), tex_coef(cc, 'x', bpn=1),
                        tex_coef(cd, '', bplus=cc != 0, bpn=1))
            else:
                return '%s%s\\,(%s%s)' % (tex_coef(ca, 'x'), tex_coef(cb,
                        '', bplus=ca != 0), tex_coef(cc, 'x'), tex_coef(cd,
                        '', bplus=cc != 0))
        elif cc == 0 or cd == 0:
            if cc == 0 and cd == 1:
                return '%s%s' % (tex_coef(ca, 'x', bplus=bplus),
                                 tex_coef(cb, '', bplus=ca != 0))
            else:
                return '(%s%s)\\times %s%s' % (tex_coef(ca, 'x'),
                        tex_coef(cb, '', bplus=ca != 0), tex_coef(cc,
                        'x', bpn=1), tex_coef(cd, '', bplus=cc != 0, bpn=
                        1))
        else:
            return '(%s%s)\\,(%s%s)' % (tex_coef(ca, 'x'), tex_coef(cb,
                    '', bplus=1), tex_coef(cc, 'x'), tex_coef(cd, '',
                    bplus=1))


def tex_dev1(a, bplus=0, bpar=0, bpn=0):  # renvoi le developpement (a)²+2*a*b+(b)², (a)²-2*a*b+(b)², (a)²-(b)² ou a*c+a*d+b*c+b*d

    # a est de la forme ((3, 2)(3, 2)) pour (3x+2)(3x+2)

    (ca, cb, cc, cd) = (a[0][0], a[0][1], a[1][0], a[1][1])  # coefficients a, b, c et d
    if a[0] == a[1]:  # (a+b)² ou (a-b)²
        if signe(ca) == signe(cb):  # (a+b)²
            (ca, cb) = (abs(ca), abs(cb))
            texte = '%s^2+2\\times %s\\times %s+%s^2' % (tex_coef(ca,
                    'x', bpc=1), tex_coef(ca, 'x', bpn=1), tex_coef(cb,
                    '', bpn=1), tex_coef(cb, '', bpn=1, bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        else:

            # (a-b)²

            (ca, cb) = (abs(ca), abs(cb))
            texte = '%s^2-2\\times %s\\times %s+%s^2' % (tex_coef(ca,
                    'x', bpc=1), tex_coef(ca, 'x', bpn=1), tex_coef(cb,
                    '', bpn=1), tex_coef(cb, '', bpn=1, bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
    if abs(ca) == abs(cc) and abs(cb) == abs(cd):  # (a+b)(a-b) ou (a+b)(-a+b)
        if ca == cc:  # (a+b)(a-b)
            texte = '%s^2-%s^2' % (tex_coef(ca, 'x', bpc=1), tex_coef(abs(cb),
                                   ''))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        else:

            # (a+b)(-a+b)

            texte = '%s^2-%s^2' % (tex_coef(cb, '', bpn=1), tex_coef(abs(ca),
                                   'x', bpc=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
    else:

        # (a+b)(c+d)

        if cc == 0 and cd == 1:
            texte = '%s%s' % (tex_coef(ca, 'x', bplus=bplus), tex_coef(cb,
                              '', bplus=ca != 0))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte
        elif ca == 0 or cb == 0 or cc == 0 or cd == 0:
            if ca == 0:
                texte = '%s\\times %s+%s\\times %s' % (tex_coef(cb, '',
                        bpn=bpn), tex_coef(cc, 'x', bpn=1), tex_coef(cb,
                        '', bpn=1), tex_coef(cd, '', bpn=1))
            elif cb == 0:
                texte = '%s\\times %s+%s\\times %s' % (tex_coef(ca, 'x',
                        bpn=bpn), tex_coef(cc, 'x', bpn=1), tex_coef(ca,
                        'x', bpn=1), tex_coef(cd, '', bpn=1))
            elif cc == 0:
                texte = '%s\\times %s+%s\\times %s' % (tex_coef(cd, '',
                        bpn=bpn), tex_coef(ca, 'x', bpn=1), tex_coef(cd,
                        '', bpn=1), tex_coef(cb, '', bpn=1))
            else:
                texte = '%s\\times %s+%s\\times %s' % (tex_coef(cc, 'x',
                        bpn=bpn), tex_coef(ca, 'x', bpn=1), tex_coef(cc,
                        'x', bpn=1), tex_coef(cb, '', bpn=1))
            return texte
        else:
            texte = '%s+%s+%s+%s' % (tex_coef(ca * cc, 'x^2', bpn=bpn),
                    tex_coef(ca * cd, 'x', bpn=1), tex_coef(cb * cc, 'x',
                    bpn=1), tex_coef(cb * cd, '', bpn=1))
            if bpar:
                return '(' + texte + ')'
            else:
                return texte


def tex_developpe1(a):  # developpe l'expression a
    (exo, cor) = ([], [])
    exo.append("\\[ \\thenocalcul = %s \\]" % tex_dev0(a))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_dev0(a))
    cor.append("\\[ \\thenocalcul = %s \\]" % tex_dev1(a))
    cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" % tex_trinome(dev(a)))
    return (exo, cor)


def tex_developpe2(a, nega, b, negb, f1, f0=None, tabs=1):  # developpe l'expression a plus l'expression b.
    if nega:
        if negb:
            if b[1][0] == 0 and b[1][1] == 1:
                if f0:
                    ecrit_tex(f0, '-' + tex_dev0(a) + '-(' + tex_dev0(b) +
                              ')', tabs=2)
                ecrit_tex(f1, '-' + tex_dev0(a) + '-(' + tex_dev0(b) +
                          ')', tabs=tabs)
            elif a[1][0] == 0 and a[1][1] == 1:
                if f0:
                    ecrit_tex(f0, '-(' + tex_dev0(a) + ')-' + tex_dev0(b),
                              tabs=2)
                ecrit_tex(f1, '-(' + tex_dev0(a) + ')-' + tex_dev0(b),
                          tabs=tabs)
            else:
                if f0:
                    ecrit_tex(f0, '-' + tex_dev0(a) + '-' + tex_dev0(b),
                              tabs=2)
                ecrit_tex(f1, '-' + tex_dev0(a) + '-' + tex_dev0(b),
                          tabs=tabs)
            ecrit_tex(f1, '-(' + tex_dev1(a) + ')-(' + tex_dev1(b) + ')',
                      tabs=tabs)
            ecrit_tex(f1, '-(' + tex_trinome(dev(a)) + ')-(' +
                      tex_trinome(dev(b)) + ')', tabs=tabs)
            ecrit_tex(f1, tex_trinome(coef_opposes(dev(a))) +
                      tex_trinome(coef_opposes(dev(b)), bplus=1), tabs=
                      tabs)
            ecrit_tex(f1, tex_trinome(somme_polynomes(coef_opposes(dev(a)),
                      coef_opposes(dev(b)))), cadre=True, tabs=tabs)
        else:
            if a[1][0] == 0 and a[1][1] == 1:
                if f0:
                    ecrit_tex(f0, '-(' + tex_dev0(a) + ')+' + tex_dev0(b),
                              tabs=2)
                ecrit_tex(f1, '-(' + tex_dev0(a) + ')+' + tex_dev0(b),
                          tabs=tabs)
                ecrit_tex(f1, '-(' + tex_dev1(a) + ')+' + tex_dev1(b,
                          bpn=1), tabs=tabs)
                ecrit_tex(f1, '-(' + tex_trinome(dev(a)) + ')+' +
                          tex_trinome(dev(b)), tabs=tabs)
            elif b[1][0] == 0 and b[1][1] == 1:
                if f0:
                    ecrit_tex(f0, '-' + tex_dev0(a) + tex_dev0(b, bplus=
                              1), tabs=2)
                ecrit_tex(f1, '-' + tex_dev0(a) + tex_dev0(b, bplus=1),
                          tabs=tabs)
                ecrit_tex(f1, '-(' + tex_dev1(a) + ')' + tex_dev1(b,
                          bplus=1), tabs=tabs)
                ecrit_tex(f1, '-(' + tex_trinome(dev(a)) + ')' +
                          tex_trinome(dev(b), bplus=1), tabs=tabs)
            else:
                if f0:
                    ecrit_tex(f0, '-' + tex_dev0(a) + '+' + tex_dev0(b),
                              tabs=2)
                ecrit_tex(f1, '-' + tex_dev0(a) + '+' + tex_dev0(b),
                          tabs=tabs)
                ecrit_tex(f1, '-(' + tex_dev1(a) + ')+' + tex_dev1(b,
                          bpn=1), tabs=tabs)
                ecrit_tex(f1, '-(' + tex_trinome(dev(a)) + ')' +
                          tex_trinome(dev(b), bplus=1), tabs=tabs)
            ecrit_tex(f1, tex_trinome(coef_opposes(dev(a))) +
                      tex_trinome(dev(b), bplus=1), tabs=tabs)
            ecrit_tex(f1, tex_trinome(somme_polynomes(coef_opposes(dev(a)),
                      dev(b))), cadre=True, tabs=tabs)
    elif negb:
        if b[1][0] == 0 and b[1][1] == 1:
            if f0:
                ecrit_tex(f0, tex_dev0(a) + '-(' + tex_dev0(b) + ')',
                          tabs=2)
            ecrit_tex(f1, tex_dev0(a) + '-(' + tex_dev0(b) + ')', tabs=
                      tabs)
        else:
            if f0:
                ecrit_tex(f0, tex_dev0(a) + '-' + tex_dev0(b), tabs=2)
            ecrit_tex(f1, tex_dev0(a) + '-' + tex_dev0(b), tabs=tabs)
        ecrit_tex(f1, tex_dev1(a) + '-(' + tex_dev1(b) + ')', tabs=tabs)
        ecrit_tex(f1, tex_trinome(dev(a)) + '-(' + tex_trinome(dev(b)) +
                  ')', tabs=tabs)
        ecrit_tex(f1, tex_trinome(dev(a)) + tex_trinome(coef_opposes(dev(b)),
                  bplus=1), tabs=tabs)
        ecrit_tex(f1, tex_trinome(somme_polynomes(dev(a), coef_opposes(dev(b)))),
                  cadre=True, tabs=tabs)
    else:
        if b[1][0] == 0 and b[1][1] == 1:
            if f0:
                ecrit_tex(f0, tex_dev0(a) + tex_dev0(b, bplus=1), tabs=2)
            ecrit_tex(f1, tex_dev0(a) + tex_dev0(b, bplus=1), tabs=tabs)
            ecrit_tex(f1, tex_dev1(a) + tex_dev1(b, bplus=1), tabs=tabs)
        else:
            if f0:
                ecrit_tex(f0, tex_dev0(a) + '+' + tex_dev0(b), tabs=2)
            ecrit_tex(f1, tex_dev0(a) + '+' + tex_dev0(b), tabs=tabs)
            ecrit_tex(f1, tex_dev1(a) + '+' + tex_dev1(b), tabs=tabs)
        ecrit_tex(f1, tex_trinome(dev(a)) + tex_trinome(dev(b), bplus=1),
                  tabs=tabs)
        ecrit_tex(f1, tex_trinome(somme_polynomes(dev(a), dev(b))),
                  cadre=True, tabs=tabs)


def tex_trinome(a, bplus=0, bpar=0):  # renvoi le trinome ax²+bx+c

    # bplus est un booleen : s'il est vrai, le premier terme est precede d'un signe + si son coefficient est positif.
    # bpar est un booleen : s'il est vrai, ecrit des parentheses autour du trinome si deux valeurs au moins ne sont pas nulles ou si la valeur est -

    texte = '%s%s%s' % (tex_coef(a[0], 'x^2', bplus=bplus), tex_coef(a[1],
                        'x', bplus=bplus or a[0] != 0), tex_coef(a[2],
                        '', bplus=bplus or a[0] != 0 or a[1] != 0))
    if bpar:
        v0 = 0
        for i in range(3):  # compte le nombre de valeurs nulles
            if a[i] == 0:
                v0 = v0 + 1
        if v0 == 2:  # une seule valeur non nulle (on suppose que les trois coef ne sont pas nuls)
            if a[0] < 0 or a[1] < 0 or a[2] < 0:  # le coefficient non nul est negatif, il faut donc des parentheses
                return '(' + texte + ')'
            else:
                return texte
        else:
            return '(' + texte + ')'
    else:
        return texte


def valeurs_apb2(pyromax):  # renvoie un tuple contenant ((3,5),(3,5))
    a = valeur_alea(1, pyromax)
    b = valeur_alea(1, pyromax)
    return ((a, b), (a, b))


def valeurs_amb2(pyromax):  # renvoie un tuple contenant ((3,-5),(3-5))
    a = valeur_alea(1, pyromax)
    b = valeur_alea(-pyromax, -1)
    return ((a, b), (a, b))


def valeurs_apbamb(pyromax):  # renvoie un tuple contenant ((3,-5),(3,+5))
    a = valeur_alea(-pyromax, pyromax)
    b = valeur_alea(-pyromax, pyromax)
    if randrange(2) == 0:
        return ((a, b), (-a, b))
    else:
        return ((a, b), (a, -b))


def valeurs_distr(pyromax):  # renvoie in tuple contenant ((a,b),(c,d)) avec a != c ou b != d (en valeur absolue) et a, b, c ou d nul.
    while True:
        a = valeur_alea(-pyromax, pyromax)
        b = valeur_alea(-pyromax, pyromax)
        c = valeur_alea(-pyromax, pyromax)
        l = [a, b, c]
        l.insert(randrange(4), 0)
        if abs(l[1]) != 1 and abs(l[3]) != 1:
            break  #Pour qu'il y ait quelque chose à développer.
    return ((l[0], l[1]), (l[2], l[3]))


def valeurs_dbldistr(pyromax):  # renvoie in tuple contenant ((a,b),(c,d)) avec a != c ou b != d (en valeur absolue)
    a = valeur_alea(-pyromax, pyromax)
    b = valeur_alea(-pyromax, pyromax)
    c = valeur_alea(-pyromax, pyromax)
    d = valeur_alea(-pyromax, pyromax)
    while abs(a) == abs(c) and abs(b) == abs(d):
        c = valeur_alea(-pyromax, pyromax)
        d = valeur_alea(-pyromax, pyromax)
    return ((a, b), (c, d))


#===============================================================================
# Développements
#===============================================================================


def def_vals():
    lexp = [0, 1, 2, 0, 1, 2]
    expr = []
    for i in range(6):
        a = valeur_alea(-10, 10)
        e = lexp.pop(randrange(len(lexp)))
        expr.append([a, 'x', e])
    return expr


def def_vals2():
    expr = []
    loper = ['+', '-', '\\times', '+', '-', '\\times', '+', '-']
    for i in range(8):
        a = valeur_alea(-10, 10)
        e = randrange(3)
        o = loper.pop(randrange(len(loper)))
        expr.append([a, 'x', e, o])
    return expr


def def_vals3():
    expr = []
    lsgn = ['+', '-', '+', '-']
    for i in range(9):
        if i % 3 == 0:
            sgn = lsgn.pop(randrange(len(lsgn)))
            if i > 0 or sgn == '-':
                expr.append(sgn)
            expr.append(" (")
        elif i % 3 == 1:
            a = valeur_alea(-10, 10)
            e = randrange(3)
            expr.append([a, 'x', e])
        else:
            a = valeur_alea(-10, 10)
            e = randrange(3)
            while e == expr[-1][2]:
                e = randrange(3)
            expr.append([a, 'x', e])
            expr.append(') ')
    return expr


def def_vals4():
    expr = []
    for i in range(9):
        a = valeur_alea(-10, 10)
        e = randrange(3)
        if i % 3 == 0:
            expr.append([a, 'x', e])
            expr.append(' \,(')
        elif i % 3 == 1:
            expr.append([a, 'x', e])
        else:
            while e == expr[-1][2]:
                e = randrange(3)
            expr.append([a, 'x', e])
            expr.append(') ')
    return expr


def def_vals5():
    expr = []
    for i in range(8):
        a = valeur_alea(-10, 10)
        e = randrange(3)
        if i % 2 == 0:
            if i % 4 == 0:
                expr.append(" (")
            else:
                expr.append(' \,(')
            expr.append([a, 'x', e])
        else:
            while e == expr[-1][2]:
                e = randrange(3)
            expr.append([a, 'x', e])
            expr.append(') ')
            if i == 3:
                expr.append(['+', '+', '-'].pop(randrange(3)))
    return expr


def monome(coef, var, exposant, bplus=0, bpn=0, bpc=0):

    # coef est le coefficient à écrire devant la variable var
    # bplus est un booleen : s'il est vrai, il faut ecrire le signe +
    # bpn est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef est negatif.
    # bpc est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef =! 0 ou 1 et var est non vide

    if exposant == 0:
        var = ''
    elif exposant > 1:
        var = '%s^{%s}' % (var, exposant)
    if coef != 0 and abs(coef) != 1:
        if var == '':
            if abs(coef) >= 1000:
                a = '\\nombre{%s}' % coef
            else:
                a = '%s' % coef
        else:
            if abs(coef) >= 1000:
                a = '\\nombre{%s}\\,%s' % (coef, var)
            else:
                a = '%s\\,%s' % (coef, var)
        if bplus and coef > 0:
            a = '+' + a
    elif coef == 1:
        if var == '':
            a = '1'
        else:
            a = '%s' % var
        if bplus:
            a = '+' + a
    elif coef == 0:
        a = ''
    elif coef == -1:
        if var == '':
            a = '-1'
        else:
            a = '-%s' % var
    if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != '':
        a = '\\left( ' + a + '\\right)'
    return a


def print_expr(expr):
    text = '$'
    for i in range(len(expr)):
        coef = expr[i][0]
        var = expr[i][1]
        exposant = expr[i][2]
        if i > 0:
            text = '%s%s' % (text, monome(coef, var, exposant, bplus=1))
        else:
            text = '%s%s' % (text, monome(coef, var, exposant))
    return '%s$' % text


def print_expr2(expr):
    text = '$'
    for i in range(len(expr)):
        coef = expr[i][0]
        var = expr[i][1]
        exposant = expr[i][2]
        if i > 0:
            text = '%s %s %s' % (text, expr[i - 1][3], monome(coef, var,
                                 exposant, bpn=1))
        else:
            text = '%s%s' % (text, monome(coef, var, exposant))
    return '%s$' % text


def print_expr3(expr):
    text = '$'
    for i in range(len(expr)):
        if isinstance(expr[i], list):
            coef = expr[i][0]
            var = expr[i][1]
            exposant = expr[i][2]
            if expr[i - 1] == " (":
                text = '%s %s' % (text, monome(coef, var, exposant))
            else:
                text = '%s %s' % (text, monome(coef, var, exposant,
                                  bplus=1))
        else:
            text = '%s%s' % (text, expr[i])
    return '%s$' % text


def print_expr4(expr):
    text = '$'
    for i in range(len(expr)):
        if isinstance(expr[i], list):
            coef = expr[i][0]
            var = expr[i][1]
            exposant = expr[i][2]
            if i == 0 or expr[i - 1] == " (":
                text = '%s %s' % (text, monome(coef, var, exposant))
            else:
                text = '%s %s' % (text, monome(coef, var, exposant,
                                  bplus=1))
        else:
            text = '%s%s' % (text, expr[i])
    return '%s$' % text



# -*- coding: utf-8 -*-

from random import randrange
from ..classes.Polynome import Fractions,Polynome, pTeX, TeX,simplifie_racine, RacineDegre2, radicalTeX, sqrt, fTeX
from ..outils.Polynomes import poly_racines_entieres, poly_racines_fractionnaires, poly_racines_quelconques, poly_id_remarquables,poly_degre3_racines_entieres, TeX_division, poly_degre3_racines_fractionnaires, randint
from ..outils.decimaux import decimaux
from ..outils.Arithmetique import pgcd
Fractions=Fractions.Fractions


def exo_racines_degre2():
    '''exercice recherche de racines second degré'''

    exo=["\\exercice"]
    cor=["\\exercice*"]
    #intervalle pour les racines entières ou fractionnaire
    rac_min=-10
    rac_max=10
    #denominateur maximmum pour les racines fractionnaires
    denom_max=denom1=12
    #Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a=1
    abs_b=10
    abs_c=10
    #X est le polynome P=x pour faciliter la construction des polynômes,
    inconnues=['x','y','z','t']
    nom_poly=['P','Q','R','S']

    exo.append(u"Résoudre les équations suivantes :")
    cor.append(u"Résoudre les équations suivantes :")
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")


    #Racines entières
    nomP='P'
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    P=poly_racines_entieres(rac_min,rac_max,X)

    exo.append("\\item $%s=0$\\par"%(P(var)))
    cor.append("\\item $%s=0$\\par"%(P(var)))
    cor=redaction_racines(P,nomP,var,cor)

    #Racines fractionnaires
    nomP='P'
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    P=poly_racines_fractionnaires(rac_min,rac_max,denom_max,X)

    exo.append("\\item $%s=0$\\par"%(P(var)))
    cor.append("\\item $%s=0$\\par"%(P(var)))
    cor=redaction_racines(P,nomP,var,cor)

    #Racines quelconques
    nomP='P'
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    P=poly_racines_quelconques(abs_a,abs_b,abs_c,X)

    exo.append("\\item $%s=0$\\par"%(P(var)))
    cor.append("\\item $%s=0$\\par"%(P(var)))
    redaction_racines(P,nomP,var,cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def exo_factorisation_degre2():
    '''exercice recherche de racines second degré'''

    exo=["\\exercice"]
    cor=["\\exercice*"]
    #intervalle pour les racines entières ou fractionnaire
    rac_min=-10
    rac_max=10

    #X est le polynome P(x)=x pour faciliter la construction des polynômes,
    inconnues=['x','y','z','t']
    nom_poly=['P','Q','R','S']

    exo.append(u"Factoriser les polynômes suivants :")
    #cor=[]u"Factoriser les polynômes suivants :"]
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")

####identites remarquables

    nomP=nom_poly[randrange(4)]
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    P,sgns=poly_id_remarquables(rac_min,rac_max,X)#sgns=-2,0 ou 2
    exo.append(u"\\item Factoriser  $%s(%s)=%s$ à l'aide d'une identité remarquable."% (nomP,var,P(var)))
    cor.append("\\item Factoriser $%s(%s)=%s$"% (nomP,var,P(var)))

    factorisation,racines=factorise_identites_remarquables(P,sgns,var,racines=True)

    factorise="$$%s"% P
    for i in range(len(factorisation)):
        factorise+="="+factorisation[i]
    cor.append(factorise+"$$")

####Racines entières
    nomP=nom_poly[randrange(4)]
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    P=poly_racines_entieres(rac_min,rac_max,X)

    exo.append("\\item $%s(%s)=%s$"%(nomP,var,P(var)))
    cor.append("\\item Factoriser $%s(%s)=%s$\\par"%(nomP,var,P(var)))
    exo,cor=redaction_factorisation(P,nomP,exo,cor)

####Racines fractionnaires
    nomP=nom_poly[randrange(4)]
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    #denominateur maximmum pour les racines fractionnaires
    denom_max=12
    P=poly_racines_fractionnaires(rac_min,rac_max,denom_max,X)

    exo.append("\\item $%s(%s)=%s$"%(nomP,var,P(var)))
    cor.append("\\item Factoriser $%s(%s)=%s$\\par"%(nomP,var,P(var)))
    exo,cor=redaction_factorisation(P,nomP,exo,cor)


####Racines quelconques
    nomP=nom_poly[randrange(4)]
    var=inconnues[randrange(4)]
    X=Polynome({1:1},var)
    #Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a=1
    abs_b=10
    abs_c=10
    P=poly_racines_quelconques(abs_a,abs_b,abs_c,X)

    exo.append("\\item $%s(%s)=%s$"%(nomP,var,P(var)))
    cor.append("\\item Factoriser $%s(%s)=%s$\\par"%(nomP,var,P(var)))
    exo,cor=redaction_factorisation(P,nomP,exo,cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def exo_factorisation_degre3():
    '''exercice de factorisation degre3'''

    #intervalle pour les racines entières ou fractionnaire
    rac_min=-10
    rac_max=10
    #denominateur maximmum pour les racines fractionnaires
    denom_max=denom1=12
    #Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a=1
    abs_b=10
    abs_c=10
    #X est le polynome P=x pour faciliter la construction des polynômes, TODO : changer  l'inconnue
    inconnues=['x','y','z','t']
    nom_poly=['P','Q','R','S']
    exo=["\\exercice",
        "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]

    X=Polynome({1:1},var="x")
    racines_quelconques=[i for i in range(-10,11)]
    E=poly_degre3_racines_entieres(rac_min,rac_max,X,racines=racines_quelconques)
    exo,cor=factorisation_degre3(E,"E",exo,cor,racines=racines_quelconques)
    F=poly_degre3_racines_fractionnaires(rac_min,rac_max,denom1,X)
    exo,cor=factorisation_degre3(F,"F",exo,cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def exo_tableau_de_signe():
    #intervalle pour les racines entières ou fractionnaire
    rac_min=-10
    rac_max=10
    #denominateur maximmum pour les racines fractionnaires
    denom_max=denom1=12
    #Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a=1
    abs_b=10
    abs_c=10
    #X est le polynome P=x pour faciliter la construction des polynômes, TODO : changer  l'inconnue
    inconnues=['x','y','z','t']
    nom_poly=['P','Q','R','S']
    borneinf=-5#float("-inf")
    bornesup=5#float("inf")
    var="x"
    X=Polynome({1:1},var=var)
    Poly=[poly_racines_entieres(rac_min,rac_max,X),
          poly_racines_fractionnaires(rac_min,rac_max,denom1,X),
          poly_racines_quelconques(abs_a,abs_b,abs_c,X)]
    intervalles=[[0,5],[-5,5],[float("-inf"),float("inf")]]
    exo=["\\exercice",
        "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]
    nomP="P"
    for i in range(len(Poly)):
        P=Poly[i]
        borneinf,bornesup=intervalles[i]
        if borneinf==float("-inf") and bornesup==float("inf"):
            TeXintervalle="\\mathbb R"
        else:
            if borneinf!=float("-inf"):
                TeXintervalle="["
            else:
                TeXintervalle="]"
            TeXintervalle+="%s~;~%s"%(TeX(borneinf),TeX(bornesup))
            if bornesup==float("inf"):
                TeXintervalle+="["
            else:
                TeXintervalle+="]"
        exo.append(u"\\item Étudier le signe du polynôme $%s=%s$ sur $I=%s$." % (nomP,P,TeXintervalle))
        cor.append(u"\\item Étudier le signe du polynôme $%s=%s$ sur $I=%s$.\\par" % (nomP,P,TeXintervalle))

        delta,simplrac,racines,str_racines,factorisation=factorisation_degre2(P,factorisation=False)
        redaction_racines(P,nomP,var,cor)
        tableau_de_signe(P,nomP,delta,racines,cor,borneinf,bornesup,detail=False)
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def exo_variation():

    exo=["\\exercice",
         "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]
    quest1,cor1=quest_fonctions_rationnelles()
    quest2,cor2=quest_variation_degre3(borneinf=-10,bornesup=10)

    exo+=quest1+quest2
    cor+=cor1  + cor2

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def exo_variation_lim():
    """Étude de fonctions avec calculs de limites"""
    exo=["\\exercice",
         "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]
    quest3,cor3=quest_variation_degre3(borneinf=float("-inf"),bornesup=float("+inf"))
    quest4,cor4=quest_fonctions_rationnelles_sur_R()
    exo+=quest3+quest4
    cor+= cor3 +cor4

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def quest_fonctions_rationnelles():

    nomf=['f','g','h','k'][randrange(4)]
    var=['t','x'][randrange(2)]
    X=Polynome({1:1},var)
    #intervalle pour les racines entières ou fractionnaire

    rac_min=-9
    rac_max=9
    b1=b2=a1=a2=0
    while b1==0 or b2==0 or a1==0 or a2==0:
        b1=randint(rac_min,rac_max)
        b2=randint(rac_min,rac_max)
        a1=randint(-5,5)
        a2=randint(-5,5)
    P=a1*X+b1
    Q=a2*X+b2

    borneinf=-10
    bornesup=10
    #Je veux que f soit définie et dérivable sur I=Intervalle
    if (-Q[0]/Q[1])>=borneinf and (-Q[0]/Q[1])<=bornesup:
        if ((-Q[0]/Q[1])-borneinf)<(bornesup-(-Q[0]/Q[1])):
            Intervalle=[int(round(-Q[0]/Q[1]))+1,bornesup]
        else:
            Intervalle=[borneinf,int(round(-Q[0]/Q[1]))-1]

    #dérivée
    numerateur="%s\\times%s-%s\\times%s"%(P.derive().TeX(parenthese=True),Q.TeX(parenthese=True),
                                          P.TeX(parenthese=True),Q.derive().TeX(parenthese=True))
    numerateur_simplifie=(P.derive()*Q-P*Q.derive()).simplifie()
    denominateur=u"%s^2"%(Q.TeX(parenthese=True))
    f_derivee="\\dfrac{%s}{%s}"%(numerateur,denominateur)
    f_derivee_simplifiee="\\dfrac{%s}{%s}"%(numerateur_simplifie,denominateur)
    VI=(-Q[0]*Fractions(1)/Q[1]).simplifie()
    exo=[u"\\item On considère la fonction $%s$ définie sur $I=[%s~;~%s]$ par $%s(%s)=\dfrac{%s}{%s}$."%(nomf,Intervalle[0],Intervalle[1],nomf,var,P,Q),
         "\\begin{enumerate}"]
    cor=[u"\\item On considère la fonction $%s$ définie sur $I=[%s~;~%s]$ par $%s(%s)=\dfrac{%s}{%s}$."%(nomf,Intervalle[0],Intervalle[1],nomf,var,P,Q),
         "\\begin{enumerate}"]

    exo.append(u"\\item Justifier que $%s$ est définie et dérivable sur $I$."%(nomf))
    cor.append(u"\\item Justifier que $%s$ est définie et dérivable sur $I$."%(nomf))
    cor.append(u" Pour déterminer la valeur interdite, on doit résoudre $%s=0$."%(Q(var)))
    cor.append("\\begin{align*}")
    cor.append("%s&=0\\\\"% Q)
    cor.append("%s&=%s"%((Q-Q[0]),TeX(-Q[0])))
    cor.append("\\\\")
    if Q[1]!=1:
        x0=-Q[0]*Fractions(1)/Q[1]
        x1=x0.simplifie()
        cor.append("%s&=%s"%(var,TeX(-Q[0]*Fractions(1)/Q[1])))
        cor.append("\\\\")
        if x0.denominateur!=x1.denominateur:
            cor.append("%s&=%s"%(var,TeX(x1)))
            cor.append("\\\\")
    cor.pop(-1)
    cor.append("\\end{align*}")
    cor.append(u"Or $%s$ n'est pas dans l'intervalle $[%s~;~%s]$ et comme $%s$ est un quotient de polynômes, alors $%s$ est définie et dérivable sur $I$."%\
               (TeX(VI),Intervalle[0],Intervalle[1],nomf,nomf))
    exo.append(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in[%s~;~%s]$."%\
          (nomf,var,var,Intervalle[0],Intervalle[1]))
    cor.append(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in[%s~;~%s]$."%\
          (nomf,var,var,Intervalle[0],Intervalle[1]))
    cor.append(u"$$%s'(%s)=%s=%s$$"%(nomf,var,f_derivee,f_derivee_simplifiee))
    exo.append(u"\\item En déduire le sens de variations de $%s$ sur $I$."%(nomf))
    cor.append(u"\\item En déduire le sens de variations de $%s$ sur $I$.\\par"%(nomf))
    if numerateur_simplifie.degre_max==0:
        cor.append(u" Comme $%s$ est un carré, il est toujours positif.\\\\"%(denominateur))
        f_xmin=TeX((Fractions(1)*P(Intervalle[0])/Q(Intervalle[0])).simplifie())
        f_xmax=TeX((Fractions(1)*P(Intervalle[1])/Q(Intervalle[1])).simplifie())
        if numerateur_simplifie[0]<0:
            cor.append(u" De plus, $%s<0$ donc pour tout $%s$ de $I$, $%s'(%s)<0$. Ainsi, on obtient "%\
                  (numerateur_simplifie[0],var,nomf,var))
            cor.append("$$\\tabvar{")
            cor.append("\\tx{%s}&\\tx{%s}&&\\tx{%s}\\cr"%(var,TeX(Intervalle[0]),TeX(Intervalle[1])))
            cor.append("\\tx{%s'(%s)}&&\\tx{-}&&\\cr"%(nomf,var))
            cor.append("\\tx{%s}&\\txh{%s}&\\fd&\\txb{%s}\\cr"%(nomf,f_xmin,f_xmax))
            cor.append("}$$")
        else:
            cor.append(u" De plus, $%s>0$ donc pour tout $%s$ de $I$, $%s'(%s)>0$."%\
                  (numerateur_simplifie[0],var,nomf,var))
            cor.append("$$\\tabvar{")
            cor.append("\\tx{%s}&\\tx{%s}&&\\tx{%s}\\cr"%(var,TeX(Intervalle[0]),TeX(Intervalle[1])))
            cor.append("\\tx{%s'(%s)}&&\\tx{+}&&\\cr"%(nomf,var))
            cor.append("\\tx{%s}&\\txb{%s}&\\fm&\\txh{%s}\\cr"%(nomf,f_xmin,f_xmax))
            cor.append("}$$")
    else:
        cor.append(u" Je ne sais pas faire avec un tel numérateur $%s$."%(numerateur_simplifie))
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def quest_fonctions_rationnelles_sur_R():

    nomf=['f','g','h','k'][randrange(4)]
    var=['t','x'][randrange(2)]
    X=Polynome({1:1},var)
    #intervalle pour les racines entières ou fractionnaire

    rac_min=-9
    rac_max=9
    b1=b2=a1=a2=0
    while b1==0 or b2==0 or a1==0 or a2==0 or a1*(-float(b2)/a1)+b1==0 or (a1*b2/a2 - b1)==0:
        #(a1*b2/a2 - b1)==0 on teste que la racine de Q n'annule pas P donc on ne peut pas simplifier
        b1=randint(rac_min,rac_max)
        b2=randint(rac_min,rac_max)
        a1=randint(-5,5)
        a2=randint(-5,5)
    P=a1*X+b1
    Q=a2*X+b2

    borneinf=float("-inf")
    bornesup=float("+inf")
    Intervalle=[borneinf,bornesup]
    TeXintervalle="\\mathbb R"

    #dérivée
    numerateur="%s\\times%s-%s\\times%s"%(P.derive().TeX(parenthese=True),Q.TeX(parenthese=True),
                                          P.TeX(parenthese=True),Q.derive().TeX(parenthese=True))
    numerateur_simplifie=(P.derive()*Q-P*Q.derive()).simplifie()
    VI=(-Q[0]*Fractions(1)/Q[1]).simplifie()

    denominateur=u"%s^2"%(Q.TeX(parenthese=True))
    f_derivee="\\dfrac{%s}{%s}"%(numerateur,denominateur)
    f_derivee_simplifiee="\\dfrac{%s}{%s}"%(numerateur_simplifie,denominateur)

    exo=[u"\\item On considère la fonction $%s$ définie  par $%s(%s)=\\dfrac{%s}{%s}$."%(nomf,nomf,var,P,Q),
         "\\begin{enumerate}"]
    cor=[u"\\item On considère la fonction $%s$ définie  par $%s(%s)=\\dfrac{%s}{%s}$."%(nomf,nomf,var,P,Q),
         "\\begin{enumerate}"]

    exo.append(u"\\item Déterminer l'ensemble de définition $\\mathcal{D}_{%s}$ de $%s$."%(nomf,nomf))
    cor.append(u"\\item Déterminer l'ensemble de définition $\\mathcal{D}_{%s}$ de $%s$.\par"%(nomf,nomf))
    cor.append(u" La fonction rationnelle $%s$ est définie et dérivable en $%s$ si $%s\\neq0$."%(nomf,var,Q(var)))
    cor.append("\\begin{align*}\n\
            %s&=0\\\\\n\
            %s&=%s\\\\\n\
            %s&=%s\\\\\n\
            %s&=%s\n\
            \\end{align*}"%(Q,(Q-Q[0]),TeX(-Q[0]),var,TeX(-Q[0]*Fractions(1)/Q[1]),var,TeX(VI)))
    cor.append(u"On en déduit que $\\mathcal{D}_{%s}=\\mathcal{D'}_{%s}=]-\\infty~;~%s[\cup]%s~;~+\\infty[$."%\
          (nomf,nomf,TeX(VI),TeX(VI)))
    exo.append(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in\\mathcal{D'}_{%s}$."%\
          (nomf,var,var,nomf))
    cor.append(u"\\item Déterminer $%s'(%s)$ pour tout $%s\in\\mathcal{D'}_{%s}$."%\
          (nomf,var,var,nomf))
    cor.append(u"$$%s'(%s)=%s=%s$$"%(nomf,var,f_derivee,f_derivee_simplifiee))
    exo.append(u"\\item Déterminer les limites de $%s$ aux bornes de $\\mathcal{D}_{%s}$."%(nomf,nomf))
    cor.append(u"\\item Déterminer les limites de $%s$ aux bornes de $\\mathcal{D}_{%s}$."%(nomf,nomf))
    limite = P[1]/Q[1]
    limite_simple = limite.simplifie()

    cor.append("$$\\lim_{%s\\to -\\infty}\\dfrac{%s}{%s}= "%(var,P,Q))

    cor.append("\\lim_{%s\\to -\\infty}\\dfrac{%s%s}{%s%s} = %s" %(var,coeffTeX(P[1]),var,coeffTeX(Q[1]),var,limite))
    if limite.n == limite_simple.n:
        cor.append("$$")
    else:
        cor.append("= %s $$"%(TeX(limite_simple)))
    cor.append("$$\\lim_{%s\\to +\\infty}\\dfrac{%s}{%s}= "%(var,P,Q))
    cor.append("\\lim_{%s\\to +\\infty}\\dfrac{%s%s}{%s%s} = %s" %(var,coeffTeX(P[1]),var,coeffTeX(Q[1]),var,limite))
    if limite.n == limite_simple.n:
        cor.append("$$")
    else:
        cor.append("= %s $$"%(TeX(limite_simple)))
    cor.append("Pour $%s=%s$, on a $%s=%s"%(var,TeX(VI),P,TeX(P(VI))))
    if P(VI)<0:
        limites=["-\\infty","+\\infty"]
        cor.append("<0$.\\\\")
    elif P(VI)>0:
        limites=["+\\infty","-\\infty"]
        cor.append(">0$.\\\\")
    else:cor.append("$.\\\\")
        #Impossible car on test (a1*b2/a2 - b1)!=0

    VIplus="\\substack{%s\\to %s\\\\%s>%s}"%(var,fTeX(VI),var,fTeX(VI))
    VImoins="\\substack{%s\\to %s\\\\%s<%s}"%(var,fTeX(VI),var,fTeX(VI))
    if Q[1]<0:
        cor.append("De plus, $%s>0$ si $%s<%s$"%(Q,var,TeX(VI)))
        cor.append("et  $%s<0$ si $%s>%s$.\\\\"%(Q,var,TeX(VI)))
        cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$"%(VImoins,P,Q,limites[0]))
        cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$"%(VIplus,P,Q,limites[1]))
    else:
        cor.append("De plus, $%s<0$ si $%s<%s$"%(Q,var,TeX(VI)))
        cor.append("et  $%s>0$ si $%s>%s$.\\\\"%(Q,var,TeX(VI)))
        cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$"%(VImoins,P,Q,limites[0]))
        cor.append(u"$$\\lim_{%s}\\dfrac{%s}{%s}=%s $$"%(VIplus,P,Q,limites[1]))

    exo.append(u"\\item Dresser le tableau de variations de $%s$ sur $\\mathcal{D}_{%s}$."%(nomf,nomf))
    cor.append(u"\\item Dresser le tableau de variations de $%s$ sur $\\mathcal{D}_{%s}$.\\par"%(nomf,nomf))
    if numerateur_simplifie.degre_max==0:
        cor.append(u" Comme $%s$ est un carré, il est toujours positif.\\\\"%(denominateur))
        f_xmin = TeX((P[1]/Q[1]).simplifie())
        f_xmax = f_xmin
        if numerateur_simplifie[0]<0:
            cor.append(u" De plus, $%s<0$ donc pour tout $%s$ de $I$, $%s'(%s)<0$. Ainsi, on obtient "%\
                  (numerateur_simplifie[0],var,nomf,var))
            cor.append("$$\\tabvar{")
            cor.append("\\tx{%s}&\\tx{%s}&&&\\tx{%s}&&&\\tx{%s}\\cr"%(var,TeX(Intervalle[0]),TeX(VI),TeX(Intervalle[1])))
            cor.append("\\tx{%s'(%s)}&&\\tx{-}&&\\dbt &&\\tx{-}&\\cr"%(nomf,var))
            cor.append("\\tx{%s}&\\txh{%s}&\\fd&\\txb{-\\infty}&\\dbt&\\txh{+\\infty}&\\fd&\\txb{%s}\\cr"%(nomf,f_xmin,f_xmax))
            cor.append("}$$")
        else:
            cor.append(u" De plus, $%s>0$ donc pour tout $%s$ de $I$, $%s'(%s)>0$."%\
                  (numerateur_simplifie[0],var,nomf,var))
            cor.append("$$\\tabvar{")
            cor.append("\\tx{%s}&\\tx{%s}&&&\\tx{%s}&&&\\tx{%s}\\cr"%(var,TeX(Intervalle[0]),TeX(VI),TeX(Intervalle[1])))
            cor.append("\\tx{%s'(%s)}&&\\tx{+}&&\\dbt&&\\tx{+}&\\cr"%(nomf,var))
            cor.append("\\tx{%s}&\\txb{%s}&\\fm&\\txh{+\\infty}&\\dbt&\\txb{-\\infty}&\\fm&\\txh{%s}\\cr"%(nomf,f_xmin,f_xmax))
            cor.append("}$$")
    else:
        cor.append(u" Je ne sais pas faire avec un tel numérateur $%s$."%(numerateur_simplifie))
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def coeffTeX(a):
        if a==1:return ""
        if a==-1:return "-"
        return a

def quest_variation_degre3(borneinf=float("-inf"),bornesup=float("+inf")):
    '''Question qui propose l'étude du sens de variation d'un polynôme de degré 3'''
    Intervalle=[borneinf,bornesup]
    if borneinf==float("-inf") and bornesup==float("+inf"):
        TeX_intervalle="\\mathbb R"
    else:
        TeX_intervalle="\\left[%s~;~%s\\right]"%(TeX(borneinf),TeX(bornesup))
    #intervalle pour les racines entières ou fractionnaire
    a=3*randint(1,3)
    rac_min=-9
    rac_max=9
    #denominateur maximmum pour les racines fractionnaires
    denom_max=denom1=12
    #Valeurs absolues maximales des coefficients d'un polynôme quelconque
    abs_a=6
    abs_b=10
    abs_c=10
    #X est le polynome P=x pour faciliter la construction des polynômes,
    inconnues=['x','y','z','t']
    nom_poly=['P','Q','R','S']
    var="x"
    X=Polynome({1:1},var=var)
    nomP=["f","g","h","k","p","q"][randrange(6)]
    Pprime=poly_racines_entieres(rac_min,rac_max,X,a1=a)
    P=Pprime.primitive()+randint(-abs_c,abs_c)
    P=P.simplifie()
    exo=[u"\\item Étudier le sens de variations de $%s$ définie par $%s(x)=%s$ sur $%s$." % (nomP,nomP,P(var),TeX_intervalle)]
    cor=[u"\\item Étudier le sens de variations de $%s$ définie par $%s(x)=%s$ sur $%s$." % (nomP,nomP,P(var),TeX_intervalle)]

    cor.append("\\par $%s'(x)=%s$\\\\" % (nomP,Pprime(var)))
    cor.append(u"Je dois étudier le signe de $%s'(%s)$ qui est un polynôme du second degré.\\par"%(nomP,var))

    delta,simplrac,racines,str_racines,factorisation=factorisation_degre2(Pprime,factorisation=False)
    #cor=redaction_factorisation(Pprime,nomP+"'",exo=[],cor=cor)[1]
    #cor.pop(-5)
    redaction_racines(Pprime,nomP+"'",var,cor)
    str_signe,str_valeurs,signes,ligne_valeurs=tableau_de_signe(Pprime,nomP+"'",delta,racines,cor,borneinf,bornesup,detail=True)

    #cor.append(tab_signe)


    if (delta<=0 and P[3]<0):
        cor+=u"Donc la fonction polynômiale $%s$ est décroissante sur $%s$." %(nomP,TeX_intervalle)
    elif (delta<=0 and P[3]>0):
        cor.append(u"Donc la fonction polynômiale $%s$ est croissante sur $%s$."%(nomP,TeX_intervalle))
    else:
        cor.append("On obtient ainsi le tableau de variation de $%s$."%nomP)
        [x1,x2]=racines
        #macro=[["txb","txh"],["fm","fd"]]

        var_de_P="\\tx{%s}& \\%s{\\rnode{neu0}{%s}}&&"%(nomP,["txb","txh"]["-"==signes[0]],TeX(P(ligne_valeurs[0])))
        compteur=0
        for i in range(0,len(signes)-1):
            if signes[i]=='+':
                if 0 and signes[i+1]=="+":
                    var_de_P+="&&"
                else:
                    compteur+=1
                    var_de_P+="\\txh{\\rnode{neu%s}{%s}}&&"%(compteur,TeX(P(ligne_valeurs[i+1])))
            else:
                if 0 and signes[i+1]=="-":
                    var_de_P+="&&"
                else:
                    compteur+=1
                    var_de_P+="\\txb{\\rnode{neu%s}{%s}}&&"%(compteur,TeX(P(ligne_valeurs[i+1])))
        compteur+=1
        if signes[-1]=="+":
            var_de_P+="\\txh{\\rnode{neu%s}{%s}}\\cr"%(compteur,TeX(P(ligne_valeurs[-1])))
        else:
            var_de_P+="\\txb{\\rnode{neu%s}{%s}}\\cr"%(compteur,TeX(P(ligne_valeurs[-1])))

        cor.append("$$ \\tabvar{\n %s\n %s\n %s}$$"%\
                   (str_valeurs,str_signe,var_de_P))
        for i in range(1,compteur+1):
            cor.append("\\ncline[nodesep=0.15,linewidth=0.5pt]{->}{neu%s}{neu%s}"%(i-1,i))
        if borneinf==float("-inf"):
            cor.append(u"$$\\lim_{%s\\to %s} %s= \\lim_{%s\\to %s} %s%s^3=%s $$  "%(var,"-\\infty",P,var,"-\\infty",P[3],var,TeX(P(float("-inf")))))
        if bornesup==float("+inf"):
            cor.append(u"$$\\lim_{%s\\to %s} %s= \\lim_{%s\\to %s} %s%s^3=%s  $$ "%(var,"+\\infty",P,var,"+\\infty",P[3],var,TeX(P(float("inf")))))
    return exo,cor


                ###############################################
                #                                             #
                #   Fonctions étudiant les polynômes          #
                #                                             #
                ###############################################

def tableau_de_signe(P,nomP,delta,racines,cor,borneinf=float("-inf"),bornesup=float("+inf"),detail=False):
    '''Étudie le signe d'un polynôme de degré2'''
    ''' ne fonctionne pas si on a pas borneinf < x1< x2 <bornesup'''
    '''detail=True permet de récupérer la dernière ligne avec le signe de P'''
    var=P.var
    signe_moinsa,signe_a=[('+','-'),('-','+')][P[2]>0] #Ca donne bien ce qu'on veut...
    signes=[]
    if delta<0:
        cor.append("Comme $\\Delta <0$, $%s(%s)$ ne s'annule pas et est toujours du signe de $a$"%(nomP,var))
        cor.append("Ainsi ")
        str_valeurs="\\tx{%s}&\\tx{%s}&& \\tx{%s}\\cr" %(var,TeX(borneinf),TeX(bornesup))
        str_signe="\\tx{%s(%s)}&&\\tx{%s}&\\cr" %(nomP,var,signe_a)
        signes=[signe_a]
        ligne_valeurs=[borneinf,bornesup]
    elif delta == 0:
        cor.append("Comme $\\Delta =0$, $%s(%s)$ s'annule une seule fois pour $%s_0=%s$ et est toujours du signe de $a$.\par"%(nomP,var,var,racines[0]))
        if racines[0]<borneinf or racines[0]>bornesup:
            if borneinf!=float("-inf"):
                intervalle="["
            else:
                intervalle="]"
            intervalle+="%s~;~%s"%(TeX(borneinf),TeX(bornesup))
            if bornesup==float("inf"):
                intervalle+="["
            else:
                intervalle+="]"
            cor.append("Or $%s$ n'est pas dans l'intervalle $%s$ donc "%(TeX(racines[0]),intervalle))
            ligne_valeurs=[borneinf,bornesup]

            str_valeurs="\\tx{%s}&\\tx{%s}&& \\tx{%s}\\cr" %(var,TeX(borneinf),TeX(bornesup))

            str_signe="\\tx{%s(%s)}&&\\tx{%s}&\\cr" %(nomP,var,signe_a)
            signes=[signe_a]
        else:

            ligne_valeurs=[borneinf,racines[0],bornesup]
            str_valeurs="\\tx{%s}&\\tx{%s}&& \\tx{%s}&& \\tx{%s}\\cr" %(var,TeX(borneinf),TeX(racines[0]),TeX(bornesup))
            str_signe="\\tx{%s(%s)}&&\\tx{%s}&\\tx{0}&\\tx{%s}&\\cr" %(nomP,var,signe_a,signe_a)
            signes=[signe_a,signe_a]
    elif delta >0:
        [x1,x2]=racines
        cor.append("Comme $\\Delta >0$, $%s(%s)$ est du signe de $-a$ entre les racines."%(nomP,var))
        compare=[(x1<=borneinf)+(x2<=borneinf),(x2>=bornesup)+(x1>=bornesup)]
        #x_x1=x_x2=sign_x1=sign_x2=""
        if compare!=[0,0]:
            if borneinf!=float("-inf"):
                intervalle="["
            else:
                intervalle="]"
            intervalle+="%s~;~%s"%(TeX(borneinf),TeX(bornesup))
            if bornesup==float("inf"):
                intervalle+="["
            else:
                intervalle+="]"

        ligne_valeurs=[borneinf]
        if compare[0]>=1 or compare[1]==2:
            x_x1=sign_x1=""
        else:
            x_x1="&\\tx{%s}&"%(TeX(x1))
            ligne_valeurs+=[x1]
            sign_x1="&\\tx{%s}&\\tx{0}"%(signe_a)
            signes+=[signe_a]

        if 2 in compare:
            entreracines="&\\tx{%s}"%(signe_a)
            signes+=[signe_a]
        else:
            entreracines="&\\tx{%s}"%(signe_moinsa)
            signes+=[signe_moinsa]

        if compare[1]>=1 or compare[0]==2:
            x_x2=sign_x2=""
        else:
            x_x2="&\\tx{%s}&"%(TeX(x2))
            ligne_valeurs+=[x2]
            sign_x2="&\\tx{0}&\\tx{%s}"%(signe_a)
            signes+=[signe_a]
        ligne_valeurs+=[bornesup]
        #Ne rien dire si une racine est égale à une borne
        compare=[compare[0]-(x1==borneinf)-(x2==borneinf),compare[1]-(x1==bornesup)-(x2==bornesup)]

        if sum(compare)==2 :
            #les deux racines sont à supprimer
            cor.append("\\par Or $%s$ et $%s$ ne sont pas dans $%s$."%(TeX(x1),TeX(x2),intervalle))

        elif compare[0]==1:
            cor.append("\\par Or $%s$ n'est pas dans $%s$."%(TeX(x1),intervalle))
        elif compare[1]==1:
            cor.append("\\par Or $%s$ n'est pas dans $%s$."%(TeX(x2),intervalle))
        cor.append("Ainsi")

        str_valeurs="\\tx{%s}& \\tx{%s}& %s %s & \\tx{%s}\\cr"%(var,TeX(borneinf),x_x1,x_x2,TeX(bornesup))

        str_signe="\\tx{%s(%s)}&%s %s %s&\\cr" %(nomP,var,sign_x1,entreracines,sign_x2)


    cor.append("$$\\tabvar{")
    cor.append(str_valeurs)
    cor.append("%s}$$"%(str_signe))
    if detail:
        return str_signe,str_valeurs,signes,ligne_valeurs

def factorise_identites_remarquables(pol1,sgns,var='',racines=True):
    '''Factorise un polynomes grâce aux identités remarquables'''
    if var=='':
        var=pol1.var
    X=Polynome({1:1},var)
    a1=pgcd(int(pol1[0]),pgcd(int(pol1[1]),int(pol1[2])))#du signe de a=pol1[2]
    if a1!=1 or a1!=-1:
        pol2=pol1/a1
    else:
        pol2=pol1
    #coeff=coeff/int(math.sqrt(a1))
    #pol2=(cx)^2 ±2× cx × b + b^2
    #pol2[2]=c^2
    #pol2[1]=2× cx × b
    #pol2[0] = b^2
    c=int(sqrt(pol2[2]))#problème d'arrondi ?

    factorisation=[]
    if a1!=1:
        pol_temp = (pol1/a1).simplifie()
        pol_temp.var = var
        factorisation.append("%s \\times\\big[ %s \\big]"%(TeX(a1),pol_temp))
        facteur2 ="%s\\times \\big["%(TeX(a1))
    else:
        facteur2 =""
    if c!=1:
        facteur2 +=u"(%s %s)^2"%(TeX(c),var)
    else:
        facteur2 +=u"%s^2"% var
    if sgns:
        if sgns==-2: #-2 => (a-b)²
            facteur2 +="-2 \\times "
        else:        #+2 => (a+b)²
            facteur2 +="+2 \\times "
        if c==1:
            facteur2 +=var
        else:
            facteur2 +=TeX(c)+var
        b=int(sqrt(pol2[0]))
        facteur2 +=" \\times %s +"%(TeX(b))
    else:
        #a²-b²
        facteur2 +="-"
        b=int(sqrt(-(pol2[0])))
    facteur2 +=u"%s^2"%(TeX(b))
    if a1!=1:
        facteur2 +="\\big]"
    factorisation.append(facteur2)
    facteur3=""
    if a1!=1:
        facteur3 +=TeX(a1)
    sgns=sgns/2
    if sgns:#(cx-b)² ou (cx+b)²
        liste_racines=[Fractions(-(sgns))*b/c]
        facteur3 +="{(%s)}^2"%(c*X+sgns*b)
    else:#(cx-b)(cx+b)
        liste_racines=[Fractions(-1)*b/c,Fractions(1)*b/c]
        facteur3 +="(%s)(%s)"%(c*X+b,c*X-b)
    factorisation.append(facteur3)
    if racines:
        return factorisation,liste_racines
    return factorisation

def racines_degre2(P):
    """renvoie les racines d'un polynôme de degré 2"""
    delta=int(P[1]**2-4*P[2]*P[0])
    if delta==0:
        x0=Fractions(-1,2)*P[1]/P[2]
        liste_racines=[x0.simplifie()]
        liste_str_racines=["\\dfrac{-%s}{2\\times %s}"%(pTeX(P[1]),pTeX(P[2]))]
        simplrac=[False]
    elif delta>0:
        simplrac,strx1,x1,strx2,x2=listeracines(P[2],P[1],delta,parentheses=False)
        liste_racines=[x1,x2]
        liste_str_racines=[strx1,strx2]
    else:
        simplrac=[False]
        liste_racines=liste_str_racines=[]
    return delta,simplrac,liste_racines,liste_str_racines
    #delta
    #simplrac[0] est True si racine de Delta se simplifie, alors simplrac[1] est la racine de delta simplifiée
    #liste_racines donne la liste des racines sous la forme simplifiée, sous forme numérique si possible
    #liste_str_racine est la liste des racines au format TeX, non simplifié.

def listeracines(a,b,delta,parentheses=False):
    '''renvoie racsimple,simplifie,formule_x1,x_1,formule_x2,x2'''
    '''avec x_1<x_2
       si parenthese=True, renvoie deux booleens
          parenthesex1=True signifie qu'il faut mettre des parenthese autour de x1
       On suppose delta >0
       simplrac est True si racine de delta se simplifie'''
    a=int(a)
    b=int(b)
    parenthesex1=parenthesex2=True #par défaut
    #simplrac=True
    strx1="\\dfrac{-%s-\\sqrt{%s}}{2\\times %s}"%(pTeX(b),TeX(delta),pTeX(a))
    strx2="\\dfrac{-%s+\\sqrt{%s}}{2\\times %s}"%(pTeX(b),TeX(delta),pTeX(a))
    ##on a strx1<strx2
    coeff,radicande=simplifie_racine(delta)

    #x1,x2 simplifiés ont une écriture fractionnaire donc
    parenthesex1=parenthesex2=False
    if coeff==1:#delta n'a pas de facteur carré, on ne peut rien simplifier
        rac_delta=radicalTeX(delta)
        simplrac=[False]
    else:
        if radicande==1:
            rac_delta=TeX(coeff)
        else:
            rac_delta=TeX(coeff)+radicalTeX(radicande)
        simplrac=[True,rac_delta]
    x1=RacineDegre2(-b,2*a,-1,delta)
    x2=RacineDegre2(-b,2*a,1,delta)
    if b==0:
        parenthesex1=(coeff*a>0)
        parenthesex2=(coeff*a<0)
    if a<0:
        strx1,strx2,x1,x2,parenthesex1,parenthesex2=strx2,strx1,x2,x1,parenthesex2,parenthesex1
    if parentheses:
        return simplrac,strx1,x1,strx2,x2,parenthesex1,parenthesex2
    else:
        return simplrac,strx1,x1,strx2,x2
        #simplrac[0] est True si racine de Delta se simplifie, alors simplrac[1] est la racine de delta simplifiée

def factorisation_degre2(P,factorisation=True):
    #x1=x2=0
    var=P.var
    X=Polynome({1:1},var)
    delta=int(P[1]**2-4*P[2]*P[0])
    if delta<0:
        factorisation=[]
        str_racines=[]
        racines=[]
        simplrac=[False]
    elif delta==0:
        x0=Fractions(-1,2)*P[1]/P[2]
        simplrac=[False]
        racines=[x0.simplifie()]
        str_racines=["\\dfrac{-%s}{2\\times %s}"%(pTeX(P[1]),pTeX(P[2]))]


        P0="%s-%s"%(var,pTeX(racines[0]))
        factorisation=[[P0,P0]]#non simplifiée

        if 0>racines[0]:
            P0=X-racines[0]
            factorisation.append([P0,P0])
    else:#delta>0
        simplrac,strx1,x1,strx2,x2=listeracines(P[2],P[1],delta)
        if isinstance(x1,RacineDegre2):
            x1=x1.simplifie()
            x2=x2.simplifie()
        racines=[x1,x2]
        str_racines=[strx1,strx2]
        P1="%s-%s"%(var,pTeX(x1))
        P2="%s-%s"%(var,pTeX(x2))
        factorisation=[[P1,P2]]#non simplifiée
        #Peut-on simplifier les parenthèses ?
        if x1.radicande==0 and (x1.numerateur<0 or x2.numerateur<0):
            P1=(X-x1)(var)
            P2=(X-x2)(var)
            factorisation.append([P1,P2])
    return delta,simplrac,racines,str_racines,factorisation


def factorisation_degre3(E,nomE,exo=[],cor=[],racines=[0,1,-1,2,-2]):
    '''Factorise un polynôme de degré 3 avec une racine évidente'''
    var=E.var
    X=Polynome({1:1},var)
    exo.append("\\item Soit $%s =%s $"%(nomE,E))
    cor.append("\\item Soit $%s=%s $"%(nomE,E))
    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")
    for x0 in racines:
        if E(x0)==0:
            break
    if x0 in [-2,-1,0,1,2]:
        exo.append(u"\\item Vérifier si $%s $ possède une racine évidente."%(nomE))
    else:
        exo.append(u"\\item Vérifier que $%s$ est une racine de $%s$."%(TeX(x0),nomE))
    exo.append("\\item Factoriser $%s $."%(nomE))
    cor.append("\\item ")

    if x0==0:
        degre_facteur=min(E.puiss)
        #degre_facteur=1
        E2=(E/(X**degre_facteur))[0]
        if degre_facteur==1:
            cor.append("On remarque que $%s$ peut se factoriser par $%s$ et $%s=%s\\left(%s\\right)$" %(nomE,var,nomE,var,E2))
        elif degre_facteur==2:
            cor.append(u"On remarque que $%s$ peut se factoriser par $%s^2$ et $%s=%s^2\\left(%s\\right)$"\
            %(nomE,E.var,nomE,E.var,E2))
            exo.append("\\end{enumerate}")
            cor.append("\\end{enumerate}")
            return exo,cor
    else:
        cor.append("Comme $%s(%s)=0$, on peut diviser $%s$ par $%s$"%(nomE,TeX(x0),nomE,X-x0))
        cor.append(TeX_division(E,(X-x0))+"")
        E2,reste=E/(X-x0)
    cor.append("\\item On doit maintenant factoriser le polynome $%s_2=%s$\\\\"%(nomE,E2))
    delta,simplrac,racines,str_racines,factorisation=factorisation_degre2(E2,factorisation=True)
    cor=redaction_factorisation(E2,nomP=nomE+"_2",exo=[],cor=cor)[1]
    cor.append("\\par")
    cor.append("On en conclue donc que $%s="%(nomE))
    final=0
    if x0==0:
        P0=E.var
    else:
        P0="\\left(%s\\right)"%(X-x0)
    if E[3]==-1:
        cor.append("-")
    elif E[3]!=1:
        cor.append(TeX(E[3]))
    if delta<0:
        P1=factorisation[-1][0]
        E_factorise="%s\\times%s$"%(P0,P1)
    elif delta==0:
        P1=factorisation[-1][0]
        E_factorise="%s\\times{\\left(%s\\right)}^2$"%(P0,P1)
    else:

        P1=factorisation[-1][0]
        P2=factorisation[-1][1]
        E_factorise="%s\\left(%s\\right)\\left(%s\\right)$"%(P0,P1,P2)
    cor.append(E_factorise)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor


#----------------- redaction ---------------------------------------------------------

def redaction_factorisation(P,nomP="P",exo=[],cor=[]):
    var=P.var
    delta,simpl_delta,racines,str_racines,factorisation=factorisation_degre2(P)
    redaction_racines(P,nomP,var,cor)

    #factorisation
    if delta<0:
        cor.append(u"On ne peut pas factoriser $%s(%s)$."%(nomP,var))
    elif delta==0:
        cor.append(u"On peut donc écrire ")
        ligne_factorisation="$$%s(%s)"%(nomP,var)
        for etape in factorisation:
            ligne_factorisation+=" = "
            if P[2]!=1:
                ligne_factorisation+="%s \\times "%(TeX(P[2]))

            ligne_factorisation+="{\\left(%s\\right)}^2"%(etape[0])
        ligne_factorisation+="$$"
        cor.append(ligne_factorisation)
    else:
        cor.append(u"On peut donc écrire ")
        ligne_factorisation="$$%s(%s)"%(nomP,var)
        for etape in factorisation:
            ligne_factorisation+=" = "
            if P[2]!=1:
                ligne_factorisation+="%s \\times "%(TeX(P[2]))
            if len(etape)==1:
                ligne_factorisation+=etape[0]
            else:
                ligne_factorisation+="\\left(%s\\right)\left(%s\\right)"%(etape[0],etape[1])
        ligne_factorisation+="$$"
        cor.append(ligne_factorisation)
    return exo,cor

def redaction_racines(P,nomP,var,cor=[]):
    delta,simpl_delta,liste_racines,liste_str_racines=racines_degre2(P)
    ligne_delta=u"Je calcule $\\Delta=%s^2-4\\times %s\\times %s=%s$"%(pTeX(P[1]),pTeX(P[2]),pTeX(P[0]),TeX(delta))
    if simpl_delta[0]:
        ligne_delta+=" et $%s=%s$.\\par"%(radicalTeX(delta),simpl_delta[1])
    else:
        ligne_delta+=".\\par"
    cor.append(ligne_delta)
    if delta<0:
        cor.append("Comme $\\Delta <0$, $%s(%s)$ n'a pas de racines."%(nomP,var))
    elif delta==0:
        cor.append("Comme $\\Delta=0$, $%s(%s)$ a une seule racine $%s_0=%s=%s$.\\par"%(nomP,var,var,liste_str_racines[0],TeX(liste_racines[0])))
    else:#delta>0
        [x1,x2]=liste_racines
        cor.append("Comme $\\Delta>0$, $%s(%s)$ a deux racines :"%(nomP,var))
        if isinstance(x1,RacineDegre2):
            simplification1=simplification2=""
            x1,detail1=x1.simplifie(True)
            x2,detail2=x2.simplifie(True)
            max_len=max(len(detail1),len(detail2))
            cor.append("\\begin{align*}")
            cor.append("%s =&%s  &%s =&%s"%\
                       (liste_str_racines[0],liste_racines[0],liste_str_racines[1],liste_racines[1]))
            cor.append("\\\\")
            for i in range(0,max_len):
                if i <len(detail1):
                    cor.append("=&%s&"%(detail1[i]))
                else:
                    cor.append("&&")
                if i <len(detail2):
                    cor.append("=&%s"%(detail2[i]))
                else:
                    cor.append("& ")
                cor.append("\\\\")
            cor.pop(-1)
            cor.append("\\end{align*}")
            cor.append("Les racines de $%s$ sont $%s_1=%s$ et $%s_2=%s$.\\par"%(nomP,var,x1,var,x2))
        else:
            [strx1,strx2]=liste_str_racines
            cor.append("Les racines de $%s$ sont $%s_1=%s=%s$ et $%s_2=%s=%s$."%(nomP,var,strx1,x1,var,strx2,x2))

    return cor




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import sys,  os,  codecs
from lxml import etree
from lxml import _elementpath as DONTUSE # Astuce pour inclure lxml dans Py2exe
from re import findall
from .TexFiles import mise_en_forme


#==============================================================
#        Gestion des extensions de fichiers
#==============================================================
def supprime_extension(filename,  ext):
    """supprime l'éventuelle extension ext du nom de fichier filename.
    ext est de la forme '.tex'"""
    if os.path.splitext(filename)[1].lower():
        return os.path.splitext(filename)[0]
    return filename

def ajoute_extension(filename,  ext):
    """ajoute si nécessaire l'extension ext au nom de fichier filename.
    ext est de la forme '.tex'"""
    if os.path.splitext(filename)[1].lower() == ext:
        return filename
    return filename + ext

#==============================================================
#        Gestion du fichier de configuration de Pyromaths
#==============================================================
def create_config_file():
    """Crée le fichier de configuration au format xml"""
    from ..Values import HOME
    root = etree.Element("pyromaths")

    child = etree.SubElement(root, "options")
    etree.SubElement(child, "nom_fichier").text="exercices"
    etree.SubElement(child, "chemin_fichier").text="%s" % HOME
    etree.SubElement(child, "titre_fiche").text=u"Fiche de révisions"
    etree.SubElement(child, "corrige").text="True"
    etree.SubElement(child, "pdf").text="True"
    etree.SubElement(child, "unpdf").text="False"
    etree.SubElement(child, "modele").text="pyromaths.tex"

    child = etree.SubElement(root, "informations")
    etree.SubElement(child, "version").text="10.10"
    etree.SubElement(child, "description").text=u"Pyromaths est un programme qui permet de générer des fiches d’exercices de mathématiques de collège ainsi que leur corrigé. Il crée des fichiers au format pdf qui peuvent ensuite être imprimés ou lus sur écran."
    etree.SubElement(child, "icone").text="pyromaths.ico"

    subchild= etree.SubElement(child, "auteur")
    etree.SubElement(subchild, "nom").text=u"Jérôme Ortais"
    etree.SubElement(subchild, "email").text=u"jerome.ortais@pyromaths.org"
    etree.SubElement(subchild, "site").text="http://www.pyromaths.org"

    return etree.tostring(root, pretty_print=True, encoding=unicode)

def indent(elem, level=0):
    """Indente correctement les fichiers xml.
    By Filip Salomonsson; published on February 06, 2007.
    http://infix.se/2007/02/06/gentlemen-indent-your-xml"""
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for e in elem:
            indent(e, level+1)
            if not e.tail or not e.tail.strip():
                e.tail = i + "  "
        if not e.tail or not e.tail.strip():
            e.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i
    return elem

def modify_config_file(file):
    """Modifie le fichier de configuration si besoin, excepté les options utilisateur déjà configurées"""
    from ..Values import CONFIGDIR
    modifie = False
    oldtree = etree.parse(file)
    oldroot = oldtree.getroot()
    newroot = etree.XML(create_config_file())
    for element in newroot.iter(tag=etree.Element):
        if not len(element):
            parents = [element]
            e = element.getparent()
            while e is not None:
                parents.insert(0,e)
                e = e.getparent()
            oldtag = oldroot
            for i in range(1, len(parents)):
                if oldtag.find(parents[i].tag) is None and i < len(parents) - 1 :
                    if i > 1:
                        etree.SubElement(oldroot.find(parents[i-1].tag), parents[i].tag)
                    else:
                        etree.SubElement(oldroot, parents[i].tag)
                    oldtag = oldtag.find(parents[i].tag)
                else:
                    oldtag = oldtag.find(parents[i].tag)
                if i == len(parents)-2: oldparent = oldtag
            if oldtag is None:
                # Ajoute un nouvel item dans le fichier xml
                modifie = True
                etree.SubElement(oldparent, element.tag).text =  element.text
            elif oldtag.text != element.text and parents[1].tag != "options":
                # Modifie un item existant s'il ne s'agit pas des options
                modifie = True
                oldtag.text =  element.text
    if modifie:
        f = codecs.open(os.path.join(CONFIGDIR, "pyromaths.xml"), encoding='utf-8', mode = 'w')
        f.write(etree.tostring(indent(oldroot), pretty_print=True, encoding=unicode))
        f.close()

#==============================================================
#        Créer et lance la compilation des fichiers TeX
#==============================================================
def creation(parametres):
    """Création et compilation des fiches d'exercices.
    parametres = {'fiche_exo': f0,
                  'fiche_cor': f1,
                  'liste_exos': self.lesexos,
                  'creer_pdf': self.checkBox_create_pdf.checkState(),
                  'creer_unpdf': self.checkBox_unpdf.isChecked() and self.checkBox_unpdf.isEnabled(),
                  'titre': unicode(self.lineEdit_titre.text()),
                  'niveau': unicode(self.comboBox_niveau.currentText()),
                }"""
    exo = unicode(parametres['fiche_exo'])
    cor = unicode(parametres['fiche_cor'])
    f0 = codecs.open(exo, encoding='utf-8', mode='w')
    f1 = codecs.open(cor, encoding='utf-8', mode='w')
    titre = parametres['titre']
    fiche_metapost = os.path.splitext(exo)[0] + '.mp'

    if parametres['creer_pdf']:
        copie_tronq_modele(f0, parametres, 'entete')
        if not parametres['creer_unpdf']:
            copie_tronq_modele(f1, parametres, 'entete')

    for exercice in parametres['liste_exos']:
        from ..sixiemes import sixiemes
        from ..cinquiemes import cinquiemes
        from ..quatriemes import quatriemes
        from ..troisiemes import troisiemes
        from ..lycee import lycee
        fonction = {0: sixiemes, 1: cinquiemes, 2: quatriemes, 3: troisiemes,
                4: lycee}
        fonction[exercice[0]].main(exercice[1], f0, f1)

    if parametres['creer_pdf']:
        if parametres['creer_unpdf']:
            f0.write("\\label{LastPage}\n")
            f0.write("\\newpage\n")
            f0.write(u"\\currentpdfbookmark{Le corrigé des exercices}{Corrigé}")
            f0.write("\\lhead{\\textsl{\\footnotesize{Page \\thepage/ \\pageref{LastCorPage}}}}\n")
            f0.write("\\setcounter{page}{1} ")
            f0.write("\\setcounter{exo}{0}\n")
            f1.write("\\label{LastCorPage}\n")
            copie_tronq_modele(f1, parametres, 'pied')
        else:
            f0.write("\\label{LastPage}\n")
            f1.write("\\label{LastPage}\n")
            copie_tronq_modele(f0, parametres, 'pied')
            copie_tronq_modele(f1, parametres, 'pied')

    f0.close()
    f1.close()

    if parametres['creer_unpdf']:
        f0 = codecs.open(exo, encoding='utf-8', mode='a')
        f1 = codecs.open(cor, encoding='utf-8', mode='r')
        for line in f1:
            f0.write(line)
        f0.close()
        f1.close()

    # indentation des fichiers teX créés
    mise_en_forme(exo)
    if parametres['corrige'] and not parametres['creer_unpdf']:
        mise_en_forme(cor)

    # Dossiers et fichiers d'enregistrement, définitions qui doivent rester avant le if suivant.
    dir0=os.path.dirname(exo)
    dir1=os.path.dirname(cor)
    import socket
    if socket.gethostname() == "sd-20841.pyromaths.org":
        # Chemin complet pour Pyromaths en ligne car pas d'accents
        f0noext=os.path.splitext(exo)[0].encode(sys.getfilesystemencoding())
        f1noext=os.path.splitext(cor)[0].encode(sys.getfilesystemencoding())
    else:
        # Pas le chemin pour les autres, au cas où il y aurait un accent dans
        # le chemin (latex ne gère pas le 8 bits)
        f0noext=os.path.splitext(os.path.basename(exo))[0].encode(sys.getfilesystemencoding())
        f1noext=os.path.splitext(os.path.basename(cor))[0].encode(sys.getfilesystemencoding())
    if parametres['creer_pdf']:
        from subprocess import call

        os.chdir(dir0)
        log = open('%s-pyromaths.log' % f0noext, 'w')
        for i in range(2):
            call(["latex", "-interaction=batchmode", "%s.tex" % f0noext], stdout=log)
        call(["dvips", "-q", "%s.dvi" % f0noext, "-o%s.ps" % f0noext], stdout=log)
        call(["ps2pdf", "-sPAPERSIZE#a4", "%s.ps" % f0noext, "%s.pdf" % f0noext], stdout=log)
        log.close()
        nettoyage(f0noext)
        if not "openpdf" in parametres or parametres["openpdf"]:
            if os.name == "nt":  #Cas de Windows.
                os.startfile('%s.pdf' % f0noext)
            elif sys.platform == "darwin":  #Cas de Mac OS X.
                os.system('open %s.pdf' % f0noext)
            else:
                os.system('xdg-open %s.pdf' % f0noext)

        if parametres['corrige'] and not parametres['creer_unpdf']:
            os.chdir(dir1)
            log = open('%s-pyromaths.log' % f1noext, 'w')
            for i in range(2):
                call(["latex", "-interaction=batchmode", "%s.tex" % f1noext], stdout=log)
            call(["dvips", "-q", "%s.dvi" % f1noext, "-o%s.ps" % f1noext], stdout=log)
            call(["ps2pdf", "-sPAPERSIZE#a4", "%s.ps" % f1noext, "%s.pdf" % f1noext], stdout=log)
            log.close()
            nettoyage(f1noext)
            if not "openpdf" in parametres or parametres["openpdf"]:
                if os.name == "nt":  #Cas de Windows.
                    os.startfile('%s.pdf' % f1noext)
                elif sys.platform == "darwin":  #Cas de Mac OS X.
                    os.system('open %s.pdf' % f1noext)
                else:
                    os.system('xdg-open %s.pdf' % f1noext)
        else:
            os.remove('%s-corrige.tex' % f0noext)

def nettoyage(basefilename):
    """Supprime les fichiers temporaires créés par LaTeX"""
    try:
        for ext in ('.aux', '.dvi', '.out', '.ps'):
            os.remove(basefilename+ext)
        if os.path.getsize('%s.pdf' % basefilename) > 1000 :
            os.remove('%s.log' % basefilename)
            os.remove('%s-pyromaths.log' % basefilename)
    except OSError:
        pass
        #le fichier à supprimer n'existe pas et on s'en moque.


def copie_tronq_modele(dest, parametres, master):
    """Copie des morceaux des modèles, suivant le schéma du master."""
    master_fin = '% fin ' + master
    master = '% ' + master
    n = 0

    ## Le fichier source doit être un modèle, donc il se trouve dans le dossier 'modeles' de pyromaths.
    source = parametres['modele']

    if os.path.isfile(os.path.join(parametres['datadir'], 'templates',source)):
        source = os.path.join(parametres['datadir'], 'templates', source)
    elif os.path.isfile(os.path.join(parametres['datadir'], 'templates', source)):
        source = os.path.join(parametres['datadir'], 'templates', source)
    else:
        #TODO: Message d'erreur, le modèle demandé n'existe pas
        print(u"Le fichier modèle n'a pas été trouvé dans %s" %
                os.path.join(parametres['datadir'], 'templates'))

    ## Les variables à remplacer :
    titre = parametres['titre']
    niveau = parametres['niveau']
    if parametres['creer_unpdf']:
        bookmark=u"\\currentpdfbookmark{Les énoncés des exercices}{Énoncés}"
    else:
        bookmark=""
    if os.name == 'nt':
        os.environ['TEXINPUTS']= os.path.normpath(os.path.join(parametres['datadir'],
            'packages'))
        tabvar = 'tabvar.tex'
    else:
        tabvar = os.path.normpath(os.path.join(parametres['datadir'],
            'packages', 'tabvar.tex'))
    #rawstring pour \tabvar -> tab + abvarsous windows
    modele = codecs.open(source, encoding='utf-8', mode='r')
    for line in modele:
        if master_fin in line:
            break
        if n > 0:
            temp = findall('##{{[A-Z]*}}##',line)
            if temp:
                occ = temp[0][4:len(temp)-5].lower()
                #line = sub('##{{[A-Z]*}}##',eval(occ),line)
                line = line.replace(temp[0], eval(occ))
            dest.write(line)

        if master in line:
            n = 1

    modele.close()
    return



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math
from random import randrange
from ..outils import Arithmetique
from ..outils import Geometrie


def eq_droites(A, B):
    (xA, yA) = A
    (xB, yB) = B
    a = ((yB - yA) * 1.0) / (xB - xA)
    b = ((xB * yA - xA * yB) * 1.0) / (xB - xA)
    return (a, b)


def inter_droites(A, B, C, D):
    """
    Calcule les coordonn\xc3\xa9es du point d'intersection des droites (AB) et (CD)
    """

    (a1, b1) = eq_droites(A, B)
    (a2, b2) = eq_droites(C, D)
    if a1 == a2:  #droites parallèles
        xI = A[0]
        yI = A[1]
    else:
        xI = ((b2 - b1) * 1.0) / (a1 - a2)
        yI = ((a1 * b2 - a2 * b1) * 1.0) / (a1 - a2)
    return (xI, yI)


def dist_pt_droite(A, B, C):
    """
    calcule la distance du point C \xc3\xa0 la droite (AB)
    """

    (a, b) = eq_droites(A, B)
    (xC, yC) = C
    d = (abs(a * xC - yC + b) * 1.0) / math.sqrt(a ** 2 + 1)
    return d


def dist_points(A, B):
    """ Calcul la distance entre deux points"""

    (xA, yA) = A
    (xB, yB) = B
    d = math.sqrt((xB - xA) ** 2 + (yB - yA) ** 2)
    return d


def coord_projete(A, B, C):
    """
    Calcule les coordonn\xc3\xa9es du projet\xc3\xa9 orthogonal de C sur la droite (AB)
    """

    (xA, yA) = A
    (xB, yB) = B
    (xC, yC) = C
    n = dist_points(A, B)
    p = (xB - xA) / n
    q = (yB - yA) / n
    s = p * (xC - xA) + q * (yC - yA)
    return (xA + s * p, yA + s * q)


def verifie_distance_mini(A, B, C, D):
    """
    V\xc3\xa9rifie que la distance minimale entre [AB] et [AC] est sup\xc3\xa9rieure \xc3\xa0 dmin
    """

    dmin = 1.2
    (xA, yA) = A
    (xB, yB) = B
    if xA > xB:
        (xA, yA, xB, yB) = (xB, yB, xA, yA)
    (xC, yC) = C
    (xD, yD) = D
    if xC > xD:
        (xC, yC, xD, yD) = (xD, yD, xC, yC)
    (xI, yI) = inter_droites(A, B, C, D)
    if xA <= xI <= xB and xC <= xI <= xD or xA <= coord_projete(A, B, C)[0] <= \
        xB and dist_pt_droite(A, B, C) < dmin or xA <= coord_projete(A,
            B, D)[0] <= xB and dist_pt_droite(A, B, D) < dmin or xC <= \
        coord_projete(C, D, A)[0] <= xD and dist_pt_droite(C, D, A) < \
        dmin or xC <= coord_projete(C, D, B)[0] <= xD and dist_pt_droite(C,
            D, B) < dmin or dist_points(A, C) < dmin or dist_points(A, D) < \
        dmin or dist_points(B, C) < dmin or dist_points(B, D) < dmin:
        isValid = False
    else:
        isValid = True
    return isValid


def verifie_angle(lpoints, A, B, C):
    """
    V\xc3\xa9rifie que l'angle BAC ne coupe pas les autres angles d\xc3\xa9j\xc3\xa0 trac\xc3\xa9s
    """

    if len(lpoints) == 0:  #Premier angle créé
        isValid = True
    else:
        for i in range(len(lpoints)):
            (A1, B1, C1) = (lpoints[i])[:3]
            isValid = verifie_distance_mini(A, B, A1, B1) and \
                verifie_distance_mini(A, B, A1, C1) and \
                verifie_distance_mini(A, C, A1, B1) and \
                verifie_distance_mini(A, C, A1, C1)
            if not isValid:
                break
    return isValid


def cree_angles(nb_angles, xmax, ymax):
    '''
    cr\xc3\xa9e une s\xc3\xa9rie d\'angles "non s\xc3\xa9quents"
    '''

    (xmax, ymax) = (xmax - .5, ymax - .5)  #taille de l'image en cm
    lg_seg = 6  #longueur des côtés des angles
    lpoints = []
    cpt = 0  #evite une boucle infinie
    while len(lpoints) < nb_angles and cpt < 1000:
        (xA, yA) = (randrange(5, xmax * 10) / 10.0, randrange(5, ymax *
                    10) / 10.0)
        alpha = randrange(360)  #angle entre un côté et l'horizontal
        if len(lpoints) < nb_angles / 2:
            beta = randrange(90, 180)  #crée un angle droit ou obtus
        else:
            beta = randrange(0, 75) + 15  #crée un angle aigu (entre 15° et 89°)
        xB = xA + lg_seg * math.cos((alpha * math.pi) / 180)
        yB = yA + lg_seg * math.sin((alpha * math.pi) / 180)
        xC = xA + lg_seg * math.cos(((alpha + beta) * math.pi) / 180)
        yC = yA + lg_seg * math.sin(((alpha + beta) * math.pi) / 180)
        (A, B, C) = ((xA, yA), (xB, yB), (xC, yC))
        if xA != xB and xA != xC and .5 < xB < xmax and .5 < yB < ymax and \
            .5 < xC < xmax and .5 < yC < ymax and verifie_angle(lpoints,
                A, B, C):
            lpoints.append((A, B, C, alpha, beta))
        else:
            cpt = cpt + 1

    #print len(lpoints)

    return lpoints


def PosAngle(alpha, beta):
    """retourne les angles pour placer les points sur la figure"""

    A = (alpha + beta / 2.0 + 180) % 360
    B = (alpha - 90) % 360
    C = (alpha + beta + 90) % 360
    return (A, B, C)


def PointName(l3noms, indice):
    list = []
    for i in range(3):
        list.append(l3noms[i])
    return tuple(list)


def figure(exo, cor, lpoints, lnoms, xmax, ymax):
    exo.append("\\begin{pspicture}(%s,%s)" % (xmax, ymax))
    exo.append("\\psframe(0,0)(%s,%s)" % (xmax, ymax))
    exo.append("\\psset{PointSymbol=none,MarkAngleRadius=0.6}")
    cor.append("\\begin{pspicture}(%s,%s)" % (xmax, ymax))
    cor.append("\\psset{PointSymbol=none,MarkAngleRadius=0.6}")
    cor.append("\\psframe(0,0)(%s,%s)" % (xmax, ymax))
    for i in range(len(lnoms)):
        points_exo = ''
        points_cor = ''
        points_exo += "\\pstGeonode[PointName={%s,%s,%s}," % lnoms[i]
        points_cor += "\\pstGeonode[PointName={%s,%s,%s}," % lnoms[i]
        points_exo += "PosAngle={%s,%s,%s}]" % PosAngle(lpoints[i][3], lpoints[i][4])
        points_cor += "PosAngle={%s,%s,%s}]" % PosAngle(lpoints[i][3], lpoints[i][4])
        for j in range(3):
            points_exo += "(%.2f,%.2f)" % lpoints[i][j]
            points_exo += "{a%s%s}" % (j, i)
            points_cor += "(%.2f,%.2f)" % lpoints[i][j]
            points_cor += "{a%s%s}" % (j, i)
        exo.append(points_exo)
        cor.append(points_cor)
        exo.append("\\pstMarkAngle{a%s%s}{a%s%s}{a%s%s}{}"%(1,i,0,i,2,i))
        cor.append("\\pstMarkAngle{a%s%s}{a%s%s}{a%s%s}{}"%(1,i,0,i,2,i))
        exo.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a1%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a1%s}" % (i, i, i, i))
        cor.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a1%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a1%s}" % (i, i, i, i))
        exo.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a2%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a2%s}" % (i, i, i, i))
        cor.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a2%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a2%s}" % (i, i, i, i))
    exo.append("\\end{pspicture}\\par")
    cor.append("\\end{pspicture}\\par")
    return (exo, cor)


def reponses(exo, cor, lpoints, lnoms):
    cor.append("\\begin{multicols}{4}")
    for i in range(len(lnoms)):
        cor.append("$\\widehat{%s%s%s}=%s\degres$\\par" % (lnoms[i][1],
                 lnoms[i][0], lnoms[i][2], lpoints[i][4]))
        if lpoints[i][4] < 90:
            cor.append("angle aigu\\par")
        elif lpoints[i][4] > 90:
            cor.append("angle obtus\\par")
        else:
            cor.append("angle droit\\par")
    cor.append("\\end{multicols}")
    exo.append("\\begin{tabularx}{\\textwidth}{|*{4}{X|}}")
    exo.append("\\hline angle 1 : & angle 2 : & angle 3 : & angle 4 : \\\\")
    exo.append("\\hline &&& \\\\ &&& \\\\ &&& \\\\ \\hline")
    exo.append("\\end{tabularx}")


def MesureAngles():
    nb_angles = 4
    (xmax, ymax) = (18, 8)  #taille de l'image en cm
    lnoms = []
    lpoints = []
    cpt = 0
    while len(lpoints) < nb_angles:
        if cpt > 1000:
            lpoints = []
            cpt = 0
        lpoints = cree_angles(nb_angles, xmax, ymax)
        cpt = cpt + 1
    tmpl = Geometrie.choix_points(3 * nb_angles)
    for i in range(nb_angles):
        lnoms.append(tuple(tmpl[3 * i:3 * i + 3]))
    exo = ["\\exercice", "Nommer, mesurer et donner la nature de chacun des angles suivants :\\par "]
    cor = ["\\exercice*", "Nommer, mesurer et donner la nature de chacun des angles suivants :\\par "]
    figure(exo, cor, lpoints, lnoms, xmax, ymax)
    reponses(exo, cor, lpoints, lnoms)
    return (exo, cor)



# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import codecs
def mise_en_forme(file):
    """
    \begin => +2 espaces si pas \begin{document}
    \item => +2 espaces après sauf si c'est \item
    \end => -2 espaces avant sauf si \end{document}
    Si longueur de ligne >80, retour ligne au dernier espace précédent et
        indentation de la ligne suivante.
    Couper la ligne après un \\begin{}[]{} ou un \\end{}
    lline : last line
    cline : current line
    """
    f = codecs.open(file, encoding='utf-8', mode='r')
    old_tex = f.readlines()
    new_tex=[]
    indent = 0
    item = False
    for cline in old_tex:
        if new_tex:
            lline = new_tex[-1]
        else:
            lline = ""
        cline = cline.strip()
        indent = trouve_indentation(cline, indent, lline)
        if indent < 0:
            print "problème"
        if cline:
            chaine, indent = traite_chaine(cline, indent)
            new_tex.extend(chaine)
        else:
            new_tex.append("")
    f.close()
    f = codecs.open(file, encoding='utf-8', mode='w')
    f.write("\n".join(new_tex))
    f.close()

def trouve_indentation(cline, indent, lline):
    if lline.find(r"\begin{")>=0:
        "indente tout ce qui suit \begin{...}"
        indent += 2
    if cline.find(r"\end{")==0:
        "desindente tout ce qui suit \end{...}"
        indent -= 2
#    if lline.find(r"\begin{enumerate}")>=0 or lline.find(r"\begin{itemize}")>=0:
#        "n'indente pas ce qui suit un environnement itemize"
#        indent -= 2
#    if cline.find(r"\item") == 0 and lline.find(r"\begin{enumerate}") < 0 \
#                                  and lline.find(r"\begin{itemize}") < 0:
#        indent -= 2
    if cline.find(r"\item") == 0 :
        indent -= 2
    if lline.find(r"\item") >= 0:
        indent += 2
    indent += compte_paires_ouvertes(lline)
    return indent

def traite_chaine(cline,  indent):
    """indente la chaine txt en fonction du paramètre indent"""
    list = []
    cline = " "*indent + cline
    list.append(cline)
    while len(list[-1]) > 80:
        if list[-1].rfind(" ", indent + 1, 80) > 0:
            for i in range(79, indent, -1):
                if list[-1][i] == " ":
                    list.append(list[-1][:i])
                    indent = trouve_indentation(list[-2][i+1:], indent, list[-1])
                    list.append(" "*indent + list[-2][i+1:])
                    list.pop(-3)
                    break
        else:
            break
    return list, indent

def trouve_paire(txt):
    """Trouve le caractère fermant pour les symboles {, [, ( qui est le 1er
    caractère de txt"""
    ouvrant = ["{", "[", "("]
    fermant = ["}", "]", ")"]
    index = ouvrant.index(txt[0])
    compte = 0
    for i in range(len(txt)):
        if txt[i] == ouvrant[index]:
            compte += 1
        elif txt[i] == fermant[index]:
            compte -= 1
        if compte == 0:
            return i
    return None

def compte_paires_ouvertes(txt):
    """Compte le nombres de paires {...}, \[...\] qui ne sont pas fermées dans
    la string txt"""
    diff = 0
    for i in ["{", r"\["]:
        diff += txt.count(i)
    for i in ["}", r"\]"]:
        diff -= txt.count(i)
    return 2*diff

#mise_en_forme("/home/jerome/Documents/projets/pyromaths/exemples/3e-corrige.tex")
#mise_en_forme("/tmp/4e.tex")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from random import randint, shuffle
from ..outils.Affichage import decimaux

def valide_hasard():
    """renvoie un nombre float non multiple de 10000"""
    nbre, unite = randint(1000,100000), randint(1,9)
    return float(nbre)*10+unite

def ArrondirNombreDecimal():
    """Crée et corrige un exercice d'arrondis avec les encadrements."""
    hasard = [valide_hasard() for i in range(4)]

    precision = [u'au millième', u'au centième', u'au dixième', u'à l\'unité',
            u'à la dizaine', u'à la centaine', 'au millier',
            u'à la dizaine de milliers']

    choix = [(i,j) for i in range(7)  for j in range(3)]
    shuffle(choix)

    choix_precision = [choix[i][0] for i in range(4)]
    choix_supinf = [choix[i][1] for i in range(4)]

##    choix_precision = [randint(0, 7), randint(0, 7), randint(0, 7),
##            randint(0, 7)]

##    choix_supinf = [randint(0, 2), randint(0, 2), randint(0, 2), randint(0, 2)]

    supinf = ['', u' par défaut', u' par excès']
#FIXME
    #Arrondir n'est pas synonyme de valeur approchée
    #Valeur approchée par excès 
    #Valeur approchée par défaut 
    #Arrondi = la « meilleure » valeur approchée
    #et ne paraît employé ici correctement
    
    nombres = [(hasard[0])/(10**(-choix_precision[0]+4)),
            (hasard[1])/(10**(-choix_precision[1]+4)),
            (hasard[2])/(10**(-choix_precision[2]+4)),
            (hasard[3])/(10**(-choix_precision[3]+4))]

    exo = ["\\exercice", '\\begin{enumerate}']
    cor = ["\\exercice*", '\\begin{enumerate}']


    for k in range(4):
        exo.append( "\\item Arrondir " + decimaux(nombres[k]) + " " +
                precision[choix_precision[k]] + supinf[choix_supinf[k]] +
                '.' )

        arrondi = round(nombres[k], -choix_precision[k]+3)

        if (arrondi > nombres[k]):
            defaut = arrondi - 10**(choix_precision[k]-3)
            exc = arrondi

        if (arrondi <= nombres[k]):
            defaut = arrondi
            exc = arrondi + 10**(choix_precision[k]-3)

        if (choix_supinf[k] == 0):
            solution = arrondi
        elif (choix_supinf[k] == 1):
            solution = defaut
        elif (choix_supinf[k] == 2):
            solution = exc

        cor.append( '\\item L\'encadrement de ' + decimaux(nombres[k]) + ' ' +
                precision[choix_precision[k]] + ' est :\\par' )
        cor.append( decimaux(defaut) + ' < ' + decimaux(nombres[k]) + ' < ' +
                decimaux(exc) + '\\par' )
        cor.append( u'On en déduit que son arrondi ' +
                precision[choix_precision[k]] + ' ' + supinf[choix_supinf[k]] +
                ' est : ' + decimaux(solution) + '.')

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")

    return (exo, cor)



# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from . import Arithmetique
import string
import math
import os
from .Conversions import radians, degres
from ..classes.Fractions import Fractions


class TeXMiseEnForme:

    def __init__(self, text):
        self.text = text

    def monome(self, coef, var, bplus=0, bpn=0, bpc=0):

        # coef est le coefficient à écrire devant la variable var
        # bplus est un booleen : s'il est vrai, il faut ecrire le signe +
        # bpn est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef est negatif.
        # bpc est un booleen : s'il est vrai, il faut mettre des parentheses autour de l'ecriture si coef =! 0 ou 1 et var est non vide

        if coef != 0 and abs(coef) != 1:
            if var == "":
                if abs(coef) >= 1000:
                    a = '\\nombre{%s}' % coef
                else:
                    a = "%s" % coef
            else:
                if abs(coef) >= 1000:
                    a = '\\nombre{%s}\\,%s' % (coef, var)
                else:
                    a = '%s\\,%s' % (coef, var)
            if bplus and coef > 0:
                a = '+' + a
        elif coef == 1:
            if var == "":
                a = '1'
            else:
                a = "%s" % var
            if bplus:
                a = '+' + a
        elif coef == 0:
            a = ""
        elif coef == -1:
            if var == "":
                a = '-1'
            else:
                a = '-%s' % var
        if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != \
            "":
            a = '\\left( ' + a + '\\right)'
        return a

    def sepmilliers(self, nb, mathenvironment=0):

        # Insère les espaces fines pour séparer les milliers et remplace le point
        # décimal par une virgule

        dec = [str(nb)[i] for i in range(len(str(nb)))]
        if dec.count('e'):  #nb ecrit en notation scientifique
            exposant = int(("").join(dec[dec.index('e') + 1:]))
            dec = dec[:dec.index('e')]
            lg = len(dec)
            if dec.count('.'):
                virg = dec.index('.')
                dec.remove('.')
            else:
                virg = len(dec)
            if virg + exposant < 0:  #L'ecriture decimale du nombre commence par 0,...
                dec2 = ["0", '.']
                for i in range(-virg - exposant):
                    dec2.append("0")
                dec2.extend(dec)
                dec = dec2
            elif virg + exposant > lg:

                #L'ecriture decimale du nombre finit par des 0

                for i in range(-((lg - virg) - 1) + exposant):
                    dec.append("0")
        dec2 = []
        if dec.count('.'):
            lavtvirg = dec.index('.')
            laprvirg = (len(dec) - dec.index('.')) - 1
        else:
            lavtvirg = len(dec)
            laprvirg = 0
        nbsep = lavtvirg // 3 + 1
        if lavtvirg > 3:
            cpt = lavtvirg % 3
            if cpt:
                dec2 = dec[0:cpt]
                dec2.append('\\,')
                nbsep = nbsep - 1
            for i in range(nbsep):
                dec2.extend(dec[cpt:cpt + 3])
                if nbsep - i > 1:
                    dec2.append('\\,')
                cpt = cpt + 3
        else:
            if dec.count('.'):
                dec2 = dec[0:dec.index('.')]
            else:
                dec2 = dec
        if dec.count('.'):
            cpt = dec.index('.')
        else:
            cpt = len(dec)
        if laprvirg <= 3:
            dec2.extend(dec[cpt:])
        else:
            nbsep = laprvirg // 3 - 1
            dec2.extend(dec[cpt:cpt + 4])
            dec2.append('\\,')
            cpt = cpt + 4
            for i in range(nbsep):
                dec2.extend(dec[cpt:cpt + 3])
                if cpt + 3 < len(dec):
                    dec2.append('\\,')
                cpt = cpt + 3
            dec2.extend(dec[cpt:])
        nb = ("").join(dec2)
        if nb.endswith('.0'):
            nb = string.rsplit(nb, '.0')[0]
        if mathenvironment:
            return string.join(string.rsplit(nb, sep='.'), '{,}')
        else:
            return string.join(string.rsplit(nb, sep='.'), ',')

sepmilliers=TeXMiseEnForme("").sepmilliers

def Affichage(l):
    """\xc3\x89crit une expressions contenant des d\xc3\xa9cimaux et des fractions au format TeX

    @param l: liste contenant l'expression [3, '+', 5, '*', 2]
    """

    expr = ""  # résultat
    for i in range(len(l)):  # on parcourt la liste
        if ["+", "-", "*", "/", '(', ')'].count(l[i]):  # Un opérateur
            if l[i] == "*":
                expr = expr + " \\times "
            elif l[i] == "/":
                expr = expr + " \\div "
            elif l[i] == ")" and type(l[i-1]) == type(Fractions(1, 1)):
                expr = expr + " \\big) "
            elif i<len(l)-2 and l[i] == "(":
                if type(l[i+1]) == type(Fractions(1, 1)):
                    expr = expr + " \\big( "
            else:
                expr = expr + " " + l[i] + " "
        elif type(l[i]) == type(Fractions(1, 1)):

            # C'est une fraction

            expr = expr + "%s" % Fractions.TeX(l[i], signe = 0)
        else:

              # C'est un nombre

            expr = expr + "%s" % Affichage.decimaux(l[i], 1)
    return expr







#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random
from ..outils.decimaux import decimaux

#
# ------------------- Aire de disques -------------------
#


def arrondir(nombre):
    [partie_entiere, partie_decimale] = nombre.split(".")
    return int(partie_entiere) + (int(partie_decimale[0])>=5)


def exo_aire_diques():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    rayon1 = 2*(random.randrange(33) + 1)
    rayon2 = int(1.5*rayon1)
    i=random.randrange(2)
    if i==0:
        donnees=('rayons', rayon1, rayon2)
    else:
        donnees=(u'diamètres', 2*rayon1, 2*rayon2)
    difference_des_carres = rayon2**2-rayon1**2
    aire_arrondie = arrondir(str(3.14 * difference_des_carres))
    enonce = \
        u"""\\begin{minipage}{4cm}
\\begin{pspicture}(-2,-2)(2,2)
\\pscircle[fillstyle=solid](0,0){1.5}
\\pscircle[fillstyle=solid, fillcolor=white](0,0){1}
\\psdots[dotstyle=x](0,0)
\\rput(0.3;60){$O$}
\\end{pspicture}
\\end{minipage}\\hfill
\\begin{minipage}{13cm}
On considère deux cercles de centre $O$ et de %s respectifs $\\unit[%s]{cm}$ et $\\unit[%s]{cm}$.\\par
Calculer l'aire de la couronne circulaire (partie colorée) comprise entre les deux cercles en arrondissant le résultat au $\\unit{cm^2}$ le plus proche.""" % donnees
    exo.append(enonce)
    cor.append(enonce)
    cor.append("\\par\\dotfill{}\\\\\n")
    if i==0:
        cor.append(u"On calcule l'aire du disque de rayon $\\unit[%s]{cm}$:" % rayon2)
        cor.append(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]" % (rayon2,  rayon2, rayon2, decimaux(rayon2**2)))
        cor.append(u"On calcule l'aire du disque de rayon $\\unit[%s]{cm}$:" % rayon1)
        cor.append(u"\\[ \\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\]" % (rayon1, rayon1, rayon1, decimaux(rayon1**2)))
    else:
        cor.append(u"Un disque de diamètre $\\unit[%s]{cm}$ a pour rayon $%s \div 2 = \\unit[%s]{cm}$. Calculons son aire:" % (2*rayon2, 2*rayon2, rayon2))
        cor.append(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]" % (rayon2,  rayon2, rayon2, decimaux(rayon2**2)))
        cor.append(u"Un disque de diamètre $\\unit[%s]{cm}$ a pour rayon $%s \div 2 = \\unit[%s]{cm}$. Calculons son aire:" % (2*rayon1, 2*rayon1, rayon1))
        cor.append(u"\\[\\pi \\times %s^2 = \\pi \\times %s \\times %s = \\unit[%s \\pi]{cm^2}\\]" % (rayon1,  rayon1, rayon1, decimaux(rayon1**2)))
    cor.append(u"L'aire $\\mathcal{A}$ de la couronne est obtenue en retranchant l'aire du disque de rayon  $\\unit[%s]{cm}$  à l'aire du disque de rayon  $\\unit[%s]{cm}$:"% (rayon1, rayon2))
    cor.append(u"\\[\\mathcal{A} = %s \\pi  - %s \\pi= (%s - %s)\\pi =\\unit[%s \\pi]{cm^2}\\]" % (decimaux(rayon2**2), decimaux(rayon1**2), decimaux(rayon2**2), decimaux(rayon1**2), decimaux(difference_des_carres)))
    cor.append(u"L'aire exacte de la couronne est $\\unit[%s \\pi]{cm^2}$." % (decimaux(difference_des_carres)))
    cor.append(u"En prenant 3,14 comme valeur approchée du nombre $\\pi$, on obtient :")
    cor.append(u"\\[\\mathcal{A}  \\approx %s \\times 3,14\\]" % decimaux(difference_des_carres))
    cor.append(u"\\[\\boxed{\\mathcal{A} \\approx  \\unit[%s]{cm^2}}\\]" % decimaux(aire_arrondie))
    exo.append("\\end{minipage}\n")
    cor.append("\\end{minipage}\n")
    return (exo, cor)
